@@ -239,7 +239,7 @@ progress_timeout_cb (RBEncoderGst *encoder)
 	if (state != GST_STATE_PLAYING)
 		return FALSE;
 
-	if (!gst_element_query_position (encoder->priv->pipeline, &format, &position)) {
+	if (!gst_element_query_position (encoder->priv->pipeline, format, &position)) {
 		g_warning ("Could not get current track position");
 		return TRUE;
 	}
@@ -303,20 +303,18 @@ add_tags_from_entry (RBEncoderGst *encoder,
 		     GError **error)
 {
 	GstTagList *tags;
+	GValue obj = {0,};
 	GstTagSetter *tag_setter;
 	GstIterator *iter;
 	gulong day;
 	gdouble bpm;
 	gboolean done;
 
-	tags = gst_tag_list_new ();
-
-	gst_tag_list_add (tags, GST_TAG_MERGE_REPLACE_ALL,
-			  GST_TAG_TRACK_NUMBER, rhythmdb_entry_get_ulong (entry, RHYTHMDB_PROP_TRACK_NUMBER),
-			  GST_TAG_ALBUM_VOLUME_NUMBER, rhythmdb_entry_get_ulong (entry, RHYTHMDB_PROP_DISC_NUMBER),
-			  GST_TAG_ENCODER, "Rhythmbox",
-			  GST_TAG_ENCODER_VERSION, VERSION,
-			  NULL);
+	tags = gst_tag_list_new (GST_TAG_TRACK_NUMBER, rhythmdb_entry_get_ulong (entry, RHYTHMDB_PROP_TRACK_NUMBER),
+				 GST_TAG_ALBUM_VOLUME_NUMBER, rhythmdb_entry_get_ulong (entry, RHYTHMDB_PROP_DISC_NUMBER),
+				 GST_TAG_ENCODER, "Rhythmbox",
+				 GST_TAG_ENCODER_VERSION, VERSION,
+				 NULL);
 
 	add_string_tag (tags, GST_TAG_MERGE_APPEND, GST_TAG_TITLE, entry, RHYTHMDB_PROP_TITLE);
 	add_string_tag (tags, GST_TAG_MERGE_APPEND, GST_TAG_ARTIST, entry, RHYTHMDB_PROP_ARTIST);
@@ -355,10 +353,12 @@ add_tags_from_entry (RBEncoderGst *encoder,
 	iter = gst_bin_iterate_all_by_interface (GST_BIN (encoder->priv->encodebin), GST_TYPE_TAG_SETTER);
 	done = FALSE;
 	while (!done) {
-		switch (gst_iterator_next (iter, (gpointer) & tag_setter)) {
+		g_value_init (&obj, GST_TYPE_ELEMENT);
+		switch (gst_iterator_next (iter, &obj)) {
 		case GST_ITERATOR_OK:
+			tag_setter = g_value_get_object (&obj);
 			gst_tag_setter_merge_tags (tag_setter, tags, GST_TAG_MERGE_REPLACE_ALL);
-			gst_object_unref (tag_setter);
+			g_value_unset (&obj);
 			break;
 		case GST_ITERATOR_RESYNC:
 			gst_iterator_resync (iter);
@@ -377,29 +377,32 @@ add_tags_from_entry (RBEncoderGst *encoder,
 }
 
 static void
-new_decoded_pad_cb (GstElement *decodebin, GstPad *new_pad, gboolean arg1, RBEncoderGst *encoder)
+pad_added_cb (GstElement *decodebin, GstPad *new_pad, RBEncoderGst *encoder)
 {
 	GstPad *enc_sinkpad;
 	GstCaps *caps;
 	gchar *caps_string;
 
-	rb_debug ("new decoded pad");
-
 	/* transcode only the first audio track. multitrack audio files are not
 	 * so common anyway */
-	if (encoder->priv->decoded_pads > 0)
+	if (encoder->priv->decoded_pads > 0) {
+		rb_debug ("already have an audio track to encode");
 		return;
+	}
 
-	caps = gst_pad_get_caps (new_pad);
+	caps = gst_pad_query_caps (new_pad, NULL);
 	caps_string = gst_caps_to_string (caps);
 	gst_caps_unref (caps);
 
 	/* only process audio data */
 	if (strncmp (caps_string, "audio/", 6) == 0) {
+		rb_debug ("linking first audio pad");
 		encoder->priv->decoded_pads++;
 		enc_sinkpad = gst_element_get_static_pad (encoder->priv->encodebin, "audio_0");
 		if (gst_pad_link (new_pad, enc_sinkpad) != GST_PAD_LINK_OK)
 			rb_debug ("error linking pads");
+	} else {
+		rb_debug ("ignoring non-audio pad");
 	}
 
 	g_free (caps_string);
@@ -414,21 +417,21 @@ add_decoding_pipeline (RBEncoderGst *encoder,
 	g_return_val_if_fail (error == NULL || *error == NULL, NULL);
 
 	encoder->priv->transcoding = TRUE;
-	decodebin = gst_element_factory_make ("decodebin2", NULL);
+	decodebin = gst_element_factory_make ("decodebin", NULL);
 	if (decodebin == NULL) {
-		rb_debug ("couldn't create decodebin2");
+		rb_debug ("couldn't create decodebin");
 		g_set_error (error,
 				RB_ENCODER_ERROR,
 				RB_ENCODER_ERROR_INTERNAL,
-				"Could not create decodebin2");
+				"Could not create decodebin");
 		return NULL;
 	}
 
 	gst_bin_add (GST_BIN (encoder->priv->pipeline), decodebin);
 
 	g_signal_connect_object (decodebin,
-			"new-decoded-pad",
-			G_CALLBACK (new_decoded_pad_cb),
+			"pad-added",
+			G_CALLBACK (pad_added_cb),
 			encoder, 0);
 
 	return decodebin;
@@ -496,7 +499,7 @@ attach_output_pipeline (RBEncoderGst *encoder,
 	}
 
 	if (sink == NULL) {
-		sink = gst_element_make_from_uri (GST_URI_SINK, dest, "sink");
+		sink = gst_element_make_from_uri (GST_URI_SINK, dest, "sink", NULL);
 		if (sink == NULL) {
 			g_set_error (error, RB_ENCODER_ERROR, RB_ENCODER_ERROR_FILE_ACCESS,
 				     _("Could not create a GStreamer sink element to write to %s"),
@@ -531,7 +534,7 @@ create_pipeline_and_source (RBEncoderGst *encoder,
 		return NULL;
 	}
 
-	src = gst_element_make_from_uri (GST_URI_SRC, uri, "source");
+	src = gst_element_make_from_uri (GST_URI_SRC, uri, "source", NULL);
 	if (src == NULL) {
 		g_set_error (error,
 			     RB_ENCODER_ERROR, RB_ENCODER_ERROR_INTERNAL,
@@ -580,7 +583,7 @@ transcode_track (RBEncoderGst *encoder,
 		 gboolean overwrite,
 		 GError **error)
 {
-	/* src ! decodebin2 ! encodebin ! sink */
+	/* src ! decodebin ! encodebin ! sink */
 	GstElement *src;
 	GstElement *decoder;
 
