@@ -43,6 +43,7 @@ typedef struct {
 	GObject *player;
 	GstElement *element;
 	GstElement *fixture;
+	gboolean linking;
 } RBGstPipelineOp;
 
 GstElement *
@@ -97,9 +98,10 @@ rb_player_gst_try_audio_sink (const char *plugin_name, const char *name)
 }
 
 static gint
-find_property_element (GstElement *element, const char *property)
+find_property_element (const GValue * value, const char *property)
 {
 	gint res = 1;
+	GstElement *element = g_value_get_object (value);
 	char *name = gst_element_get_name (element);
 
 	if (g_object_class_find_property (G_OBJECT_GET_CLASS (element), property) != NULL) {
@@ -107,7 +109,6 @@ find_property_element (GstElement *element, const char *property)
 		return 0;
 	} else {
 		rb_debug ("didn't find property \"%s\" on element %s", property, name);
-		g_object_unref (element);
 	}
 		
 	g_free (name);
@@ -118,7 +119,8 @@ GstElement *
 rb_player_gst_find_element_with_property (GstElement *element, const char *property)
 {
 	GstIterator *iter;
-	GstElement *result;
+	GValue r = {0,};
+	GstElement *result = NULL;
 
 	if (GST_IS_BIN (element) == FALSE) {
 		if (g_object_class_find_property (G_OBJECT_GET_CLASS (element),
@@ -130,9 +132,13 @@ rb_player_gst_find_element_with_property (GstElement *element, const char *prope
 
 	rb_debug ("iterating bin looking for property %s", property);
 	iter = gst_bin_iterate_recurse (GST_BIN (element));
-	result = gst_iterator_find_custom (iter,
-					   (GCompareFunc) find_property_element,
-					   (gpointer) property);
+	if (gst_iterator_find_custom (iter,
+				      (GCompareFunc) find_property_element,
+				      &r,
+				      (gpointer) property)) {
+		result = g_value_dup_object (&r);
+		g_value_unset (&r);
+	}
 	gst_iterator_free (iter);
 	return result;
 }
@@ -145,12 +151,13 @@ rb_player_gst_find_element_with_property (GstElement *element, const char *prope
  * Converts embedded image data extracted from a tag list into
  * a #GdkPixbuf.  The returned #GdkPixbuf is owned by the caller.
  *
- * Returns: a #GdkPixbuf, or NULL.
+ * Returns: (transfer full): a #GdkPixbuf, or NULL.
  */
 GdkPixbuf *
 rb_gst_process_embedded_image (const GstTagList *taglist, const char *tag)
 {
-	GstBuffer *buf;
+	GstSample *sample;
+	GstMapInfo mapinfo;
 	GdkPixbufLoader *loader;
 	GdkPixbuf *pixbuf;
 	GError *error = NULL;
@@ -161,20 +168,20 @@ rb_gst_process_embedded_image (const GstTagList *taglist, const char *tag)
 		const GValue *value;
 		const char *media_type;
 		GstStructure *caps_struct;
-		int imgtype;
+		int imgtype = 0;
 
 		value = gst_tag_list_get_value_index (taglist, tag, i);
 		if (value == NULL) {
 			break;
 		}
 
-		buf = gst_value_get_buffer (value);
-		if (buf == NULL) {
-			rb_debug ("apparently couldn't get image buffer");
+		sample = gst_value_get_sample (value);
+		if (sample == NULL) {
+			rb_debug ("apparently couldn't get image sample");
 			continue;
 		}
 
-		caps_struct = gst_caps_get_structure (buf->caps, 0);
+		caps_struct = gst_caps_get_structure (gst_sample_get_caps (sample), 0);
 
 		media_type = gst_structure_get_name (caps_struct);
 		/* other things to ignore? */
@@ -204,10 +211,13 @@ rb_gst_process_embedded_image (const GstTagList *taglist, const char *tag)
 	rb_debug ("found image at value %u for tag %s", i, tag);
 
 	loader = gdk_pixbuf_loader_new ();
-	buf = gst_value_get_buffer (val);
-	rb_debug ("sending %d bytes to pixbuf loader", buf->size);
-	if (gdk_pixbuf_loader_write (loader, buf->data, buf->size, &error) == FALSE) {
+	sample = gst_value_get_sample (val);
+	gst_buffer_map (gst_sample_get_buffer (sample), &mapinfo, GST_MAP_READ);
+	rb_debug ("sending %" G_GSIZE_FORMAT " bytes to pixbuf loader", mapinfo.size);
+
+	if (gdk_pixbuf_loader_write (loader, mapinfo.data, mapinfo.size, &error) == FALSE) {
 		rb_debug ("pixbuf loader doesn't like the data: %s", error->message);
+		gst_buffer_unmap (gst_sample_get_buffer (sample), &mapinfo);
 		g_error_free (error);
 		g_object_unref (loader);
 		return NULL;
@@ -218,6 +228,7 @@ rb_gst_process_embedded_image (const GstTagList *taglist, const char *tag)
 		g_object_ref (pixbuf);
 	}
 	
+	gst_buffer_unmap (gst_sample_get_buffer (sample), &mapinfo);
 	gdk_pixbuf_loader_close (loader, NULL);
 	g_object_unref (loader);
 
@@ -266,6 +277,8 @@ rb_gst_process_tag_string (const GstTagList *taglist,
 		*field = RB_METADATA_FIELD_GENRE;
 	else if (!strcmp (tag, GST_TAG_COMMENT))
 		*field = RB_METADATA_FIELD_COMMENT;
+	else if (!strcmp (tag, GST_TAG_ORGANIZATION))
+		*field = RB_METADATA_FIELD_ORGANIZATION;
 	else if (!strcmp (tag, GST_TAG_BITRATE))
 		*field = RB_METADATA_FIELD_BITRATE;
 	else if (!strcmp (tag, GST_TAG_MUSICBRAINZ_TRACKID))
@@ -328,7 +341,6 @@ rb_gst_error_get_error_code (const GError *error)
 }
 
 /* pipeline block-add/remove-unblock operations */
-
 static RBGstPipelineOp *
 new_pipeline_op (GObject *player, GstElement *fixture, GstElement *element)
 {
@@ -349,43 +361,19 @@ free_pipeline_op (RBGstPipelineOp *op)
 	g_free (op);
 }
 
-static void
-pipeline_op_done (GstPad *pad, gboolean blocked, GstPad *new_pad)
-{
-	GstEvent *segment;
-	if (new_pad == NULL)
-		return;
-
-	/* send a very unimaginative new segment through the new pad */
-	segment = gst_event_new_new_segment (TRUE,
-					     1.0,
-					     GST_FORMAT_DEFAULT,
-					     0,
-					     GST_CLOCK_TIME_NONE,
-					     0);
-	gst_pad_send_event (new_pad, segment);
-	gst_object_unref (new_pad);
-}
-
 static gboolean
 pipeline_op (GObject *player,
 	     GstElement *fixture,
 	     GstElement *element,
 	     gboolean use_pad_block,
-	     GstPadBlockCallback callback)
+	     GstPadProbeCallback callback)
 {
 	RBGstPipelineOp *op;
-	GstPad *fixture_pad;
 	GstPad *block_pad;
 
 	op = new_pipeline_op (player, fixture, element);
 
-	/* seems like we should be able to just block the src pad connected
-	 * to the fixture's sink pad..
-	 */
-	fixture_pad = gst_element_get_static_pad (fixture, "sink");
-	block_pad = gst_pad_get_peer (fixture_pad);
-	gst_object_unref (fixture_pad);
+	block_pad = gst_element_get_static_pad (fixture, "sink");
 
 	if (use_pad_block) {
 		char *whatpad;
@@ -393,13 +381,10 @@ pipeline_op (GObject *player,
 		rb_debug ("blocking pad %s to perform an operation", whatpad);
 		g_free (whatpad);
 
-		gst_pad_set_blocked_async (block_pad,
-					   TRUE,
-					   callback,
-					   op);
+		gst_pad_add_probe (block_pad, GST_PAD_PROBE_TYPE_BLOCK_DOWNSTREAM, callback, op, NULL);
 	} else {
 		rb_debug ("not using pad blocking, calling op directly");
-		(*callback) (block_pad, FALSE, op);
+		(*callback) (block_pad, NULL, op);
 	}
 
 	gst_object_unref (block_pad);
@@ -408,6 +393,8 @@ pipeline_op (GObject *player,
 
 /* RBPlayerGstFilter implementation */
 
+
+
 /**
  * rb_gst_create_filter_bin:
  *
@@ -429,22 +416,22 @@ rb_gst_create_filter_bin ()
 	audioconvert = gst_element_factory_make ("audioconvert", "filteraudioconvert");
 	identity = gst_element_factory_make ("identity", "filteridentity");
 	gst_bin_add_many (GST_BIN (bin), audioconvert, identity, NULL);
-	gst_element_link (audioconvert, identity);
+	gst_element_link (identity, audioconvert);
 
-	pad = gst_element_get_static_pad (audioconvert, "sink");
+	pad = gst_element_get_static_pad (identity, "sink");
 	gst_element_add_pad (bin, gst_ghost_pad_new ("sink", pad));
 	gst_object_unref (pad);
 
-	pad = gst_element_get_static_pad (identity, "src");
+	pad = gst_element_get_static_pad (audioconvert, "src");
 	gst_element_add_pad (bin, gst_ghost_pad_new ("src", pad));
 	gst_object_unref (pad);
 
 	return bin;
 }
 
-static void
+static GstPadProbeReturn
 really_add_filter (GstPad *pad,
-		   gboolean blocked,
+		   GstPadProbeInfo *info,
 		   RBGstPipelineOp *op)
 {
 	GstPad *binsinkpad;
@@ -452,26 +439,22 @@ really_add_filter (GstPad *pad,
 	GstPad *realpad;
 	GstPad *prevpad;
 	GstElement *bin;
-	GstElement *identity;
 	GstElement *audioconvert;
-	GstElement *audioconvert2;
 	GstPadLinkReturn link;
 
-	rb_debug ("adding filter %p", op->element);
+	if (op->linking) {
+		rb_debug ("still going");
+		return GST_PAD_PROBE_OK;
+	}
+	op->linking = TRUE;
 
-	/*
-	 * it kind of looks like we need audioconvert elements on either side of each filter
-	 * to prevent caps renegotiation from causing 'internal data flow error' errors.
-	 * this probably means we'd be doing a few unnecessary conversions when there are
-	 * multiple filters in the pipeline, but at least it works.
-	 */
+	rb_debug ("adding filter %p", op->element);
 
 	/* create containing bin */
 	bin = gst_bin_new (NULL);
 	audioconvert = gst_element_factory_make ("audioconvert", NULL);
-	audioconvert2 = gst_element_factory_make ("audioconvert", NULL);
-	gst_bin_add_many (GST_BIN (bin), audioconvert, op->element, audioconvert2, NULL);
-	gst_element_link_many (audioconvert, op->element, audioconvert2, NULL);
+	gst_bin_add_many (GST_BIN (bin), audioconvert, op->element, NULL);
+	gst_element_link_many (audioconvert, op->element, NULL);
 
 	/* create ghost pads */
 	realpad = gst_element_get_static_pad (audioconvert, "sink");
@@ -479,17 +462,17 @@ really_add_filter (GstPad *pad,
 	gst_element_add_pad (bin, binsinkpad);
 	gst_object_unref (realpad);
 
-	realpad = gst_element_get_static_pad (audioconvert2, "src");
+	realpad = gst_element_get_static_pad (op->element, "src");
 	binsrcpad = gst_ghost_pad_new ("src", realpad);
 	gst_element_add_pad (bin, binsrcpad);
 	gst_object_unref (realpad);
 
 	/* chuck it into the filter bin */
 	gst_bin_add (GST_BIN (op->fixture), bin);
-	identity = gst_bin_get_by_name (GST_BIN (op->fixture), "filteridentity");
-	realpad = gst_element_get_static_pad (identity, "sink");
+	audioconvert = gst_bin_get_by_name (GST_BIN (op->fixture), "filteraudioconvert");
+	realpad = gst_element_get_static_pad (audioconvert, "sink");
 	prevpad = gst_pad_get_peer (realpad);
-	gst_object_unref (identity);
+	gst_object_unref (audioconvert);
 
 	gst_pad_unlink (prevpad, realpad);
 
@@ -504,7 +487,10 @@ really_add_filter (GstPad *pad,
 		gst_object_unref (bin);
 
 		free_pipeline_op (op);
-		return;
+		if (info) {
+			gst_pad_remove_probe (pad, info->id);
+		}
+		return GST_PAD_PROBE_OK;
 	}
 
 	link = gst_pad_link (binsrcpad, realpad);
@@ -513,34 +499,43 @@ really_add_filter (GstPad *pad,
 		g_warning ("couldn't link new filter into pipeline (src): %d", link);
 		/* doubt we can do anything helpful here.. */
 	}
-
+	
 	/* if we're supposed to be playing, unblock the sink */
-	if (blocked) {
-		rb_debug ("unblocking pad after adding filter");
+	if (info) {
 		gst_element_set_state (bin, GST_STATE_PLAYING);
-		gst_pad_set_blocked_async (pad, FALSE, (GstPadBlockCallback)pipeline_op_done, NULL);
+		gst_pad_remove_probe (pad, info->id);
 	} else {
 		gst_element_set_state (bin, GST_STATE_PAUSED);
 	}
-
+	
+	rb_debug ("filter added");
 	_rb_player_gst_filter_emit_filter_inserted (RB_PLAYER_GST_FILTER (op->player), op->element);
-
 	free_pipeline_op (op);
+	return GST_PAD_PROBE_OK;
 }
 
-static void
+static GstPadProbeReturn
 really_remove_filter (GstPad *pad,
-		      gboolean blocked,
+		      GstPadProbeInfo *info,
 		      RBGstPipelineOp *op)
 {
 	GstPad *mypad;
 	GstPad *prevpad, *nextpad;
 	GstElement *bin;
 
+	if (op->linking) {
+		rb_debug ("still going");
+		return GST_PAD_PROBE_OK;
+	}
+	op->linking = TRUE;
+
 	/* get the containing bin and remove it */
 	bin = GST_ELEMENT (gst_element_get_parent (op->element));
 	if (bin == NULL) {
-		return;
+		if (info) {
+			gst_pad_remove_probe (pad, info->id);
+		}
+		return GST_PAD_PROBE_OK;
 	}
 
 	rb_debug ("removing filter %p", op->element);
@@ -570,13 +565,12 @@ really_remove_filter (GstPad *pad,
 	gst_bin_remove (GST_BIN (op->fixture), bin);
 	gst_object_unref (bin);
 
-	/* if we're supposed to be playing, unblock the sink */
-	if (blocked) {
-		rb_debug ("unblocking pad after removing filter");
-		gst_pad_set_blocked_async (pad, FALSE, (GstPadBlockCallback)pipeline_op_done, NULL);
+	rb_debug ("filter removed");
+	if (info) {
+		gst_pad_remove_probe (pad, info->id);
 	}
-
 	free_pipeline_op (op);
+	return GST_PAD_PROBE_OK;
 }
 
 /**
@@ -598,7 +592,7 @@ rb_gst_add_filter (RBPlayer *player, GstElement *filterbin, GstElement *element,
 			    filterbin,
 			    element,
 			    use_pad_block,
-			    (GstPadBlockCallback) really_add_filter);
+			    (GstPadProbeCallback) really_add_filter);
 }
 
 /**
@@ -620,13 +614,13 @@ rb_gst_remove_filter (RBPlayer *player, GstElement *filterbin, GstElement *eleme
 			    filterbin,
 			    element,
 			    use_pad_block,
-			    (GstPadBlockCallback) really_remove_filter);
+			    (GstPadProbeCallback) really_remove_filter);
 }
 
 /* RBPlayerGstTee implementation */
 
-static void
-really_add_tee (GstPad *pad, gboolean blocked, RBGstPipelineOp *op)
+static GstPadProbeReturn
+really_add_tee (GstPad *pad, GstPadProbeInfo *info, RBGstPipelineOp *op)
 {
 	GstElement *queue;
 	GstElement *audioconvert;
@@ -634,6 +628,10 @@ really_add_tee (GstPad *pad, gboolean blocked, RBGstPipelineOp *op)
 	GstElement *parent_bin;
 	GstPad *sinkpad;
 	GstPad *ghostpad;
+	
+	if (op->linking)
+		return GST_PAD_PROBE_PASS;
+	op->linking = TRUE;
 
 	rb_debug ("really adding tee %p", op->element);
 
@@ -663,33 +661,29 @@ really_add_tee (GstPad *pad, gboolean blocked, RBGstPipelineOp *op)
 	gst_bin_add (GST_BIN (parent_bin), bin);
 	gst_element_link (op->fixture, bin);
 
-	/* if we're supposed to be playing, unblock the sink */
-	if (blocked) {
-		rb_debug ("unblocking pad after adding tee");
-
+	/* try to set its state appropriately */
+	if (info) {
 		gst_element_set_state (parent_bin, GST_STATE_PLAYING);
-		gst_object_ref (ghostpad);
-		gst_pad_set_blocked_async (pad,
-					   FALSE,
-					   (GstPadBlockCallback)pipeline_op_done,
-					   ghostpad);
+		gst_pad_remove_probe (pad, info->id);
 	} else {
 		gst_element_set_state (bin, GST_STATE_PAUSED);
-		gst_object_ref (ghostpad);
-		pipeline_op_done (NULL, FALSE, ghostpad);
 	}
 
 	_rb_player_gst_tee_emit_tee_inserted (RB_PLAYER_GST_TEE (op->player), op->element);
-
 	free_pipeline_op (op);
+	return GST_PAD_PROBE_PASS;
 }
 
-static void
-really_remove_tee (GstPad *pad, gboolean blocked, RBGstPipelineOp *op)
+static GstPadProbeReturn
+really_remove_tee (GstPad *pad, GstPadProbeInfo *info, RBGstPipelineOp *op)
 {
 	GstElement *bin;
 	GstElement *parent;
 
+	if (op->linking)
+		return GST_PAD_PROBE_OK;
+	op->linking = TRUE;
+
 	rb_debug ("really removing tee %p", op->element);
 
 	_rb_player_gst_tee_emit_tee_pre_remove (RB_PLAYER_GST_TEE (op->player), op->element);
@@ -705,13 +699,11 @@ really_remove_tee (GstPad *pad, gboolean blocked, RBGstPipelineOp *op)
 	gst_bin_remove (GST_BIN (bin), op->element);
 	g_object_unref (bin);
 
-	/* if we're supposed to be playing, unblock the sink */
-	if (blocked) {
-		rb_debug ("unblocking pad after removing tee");
-		gst_pad_set_blocked_async (pad, FALSE, (GstPadBlockCallback)pipeline_op_done, NULL);
+	if (info) {
+		gst_pad_remove_probe (pad, info->id);
 	}
-
 	free_pipeline_op (op);
+	return GST_PAD_PROBE_OK;
 }
 
 /**
@@ -733,7 +725,7 @@ rb_gst_add_tee (RBPlayer *player, GstElement *tee, GstElement *element, gboolean
 			    tee,
 			    element,
 			    use_pad_block,
-			    (GstPadBlockCallback) really_add_tee);
+			    (GstPadProbeCallback) really_add_tee);
 }
 
 /**
@@ -755,6 +747,5 @@ rb_gst_remove_tee (RBPlayer *player, GstElement *tee, GstElement *element, gbool
 			    tee,
 			    element,
 			    use_pad_block,
-			    (GstPadBlockCallback) really_remove_tee);
+			    (GstPadProbeCallback) really_remove_tee);
 }
-
