@@ -35,10 +35,6 @@
 #include <glib/gi18n.h>
 #include <glib/gstdio.h>
 
-#ifdef WITH_GNOME_KEYRING
-#include <gnome-keyring.h>
-#endif
-
 #include <totem-pl-parser.h>
 
 #include "rb-audioscrobbler-radio-source.h"
@@ -170,9 +166,6 @@ struct _RBAudioscrobblerRadioSourcePrivate
 	GtkWidget *error_info_bar;
 	GtkWidget *error_info_bar_label;
 
-	GtkWidget *password_info_bar;
-	GtkWidget *password_info_bar_entry;
-
 	RBEntryView *track_view;
 	RhythmDBQueryModel *track_model;
 
@@ -184,16 +177,6 @@ struct _RBAudioscrobblerRadioSourcePrivate
 	RhythmDBEntry *playing_entry;
 
 	RBExtDB *art_store;
-
-	guint ui_merge_id;
-	GtkActionGroup *action_group;
-
-	/* used when streaming radio using old api */
-	char *old_api_password;
-	char *old_api_session_id;
-	char *old_api_base_url;
-	char *old_api_base_path;
-	gboolean old_api_is_banned;
 };
 
 #define RB_AUDIOSCROBBLER_RADIO_SOURCE_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), RB_TYPE_AUDIOSCROBBLER_RADIO_SOURCE, RBAudioscrobblerRadioSourcePrivate))
@@ -228,41 +211,21 @@ static void xspf_entry_parsed (TotemPlParser *parser,
                                GHashTable *metadata,
                                RBAudioscrobblerRadioSource *source);
 
-/* old api */
-static void old_api_shake_hands (RBAudioscrobblerRadioSource *source);
-static void old_api_handshake_response_cb (SoupSession *session,
-                                           SoupMessage *msg,
-                                           gpointer user_data);
-static void old_api_tune (RBAudioscrobblerRadioSource *source);
-static void old_api_tune_response_cb (SoupSession *session,
-                                      SoupMessage *msg,
-                                      gpointer user_data);
-static void old_api_fetch_playlist (RBAudioscrobblerRadioSource *source);
-
 /* info bar related things */
 static void display_error_info_bar (RBAudioscrobblerRadioSource *source,
                                     const char *message);
-static void display_password_info_bar (RBAudioscrobblerRadioSource *source);
-static void password_info_bar_response_cb (GtkInfoBar *info_bar,
-                                           int response_id,
-                                           RBAudioscrobblerRadioSource *source);
-
-/* action callbacks */
-static void rename_station_action_cb (GtkAction *action,
-                                      RBAudioscrobblerRadioSource *source);
-static void delete_station_action_cb (GtkAction *action,
-                                      RBAudioscrobblerRadioSource *source);
-
 
 /* RBDisplayPage implementations */
 static void impl_selected (RBDisplayPage *page);
 static void impl_get_status (RBDisplayPage *page, char **text, char **progress_text, float *progress);
-static gboolean impl_show_popup (RBDisplayPage *page);
 static void impl_delete_thyself (RBDisplayPage *page);
+static gboolean impl_can_remove (RBDisplayPage *page);
+static void impl_remove (RBDisplayPage *page);
 
 /* RBSource implementations */
 static RBEntryView *impl_get_entry_view (RBSource *asource);
 static RBSourceEOFType impl_handle_eos (RBSource *asource);
+static void impl_get_playback_status (RBSource *source, char **text, float *progress);
 
 enum {
 	PROP_0,
@@ -274,18 +237,6 @@ enum {
 	PROP_PLAY_ORDER
 };
 
-#define AUDIOSCROBBLER_RADIO_SOURCE_POPUP_PATH "/AudioscrobblerRadioSourcePopup"
-
-static GtkActionEntry rb_audioscrobbler_radio_source_actions [] =
-{
-	{ "AudioscrobblerRadioRenameStation", NULL, N_("_Rename Station"), NULL,
-	  N_("Rename station"),
-	  G_CALLBACK (rename_station_action_cb) },
-	{ "AudioscrobblerRadioDeleteStation", GTK_STOCK_DELETE, N_("_Delete Station"), NULL,
-	  N_("Delete station"),
-	  G_CALLBACK (delete_station_action_cb) }
-};
-
 G_DEFINE_DYNAMIC_TYPE (RBAudioscrobblerRadioSource, rb_audioscrobbler_radio_source, RB_TYPE_STREAMING_SOURCE)
 
 RBSource *
@@ -300,7 +251,7 @@ rb_audioscrobbler_radio_source_new (RBAudioscrobblerProfilePage *parent,
 	RBShell *shell;
 	GObject *plugin;
 	RhythmDB *db;
-	char *toolbar_path;
+	GMenu *toolbar_menu;
 
 	g_object_get (parent, "shell", &shell, "plugin", &plugin, NULL);
 	g_object_get (shell, "db", &db, NULL);
@@ -309,7 +260,7 @@ rb_audioscrobbler_radio_source_new (RBAudioscrobblerProfilePage *parent,
 		rb_audioscrobbler_radio_track_register_entry_type (db);
 	}
 
-	g_object_get (parent, "toolbar-path", &toolbar_path, NULL);
+	g_object_get (parent, "toolbar-menu", &toolbar_menu, NULL);
 
 	source = g_object_new (RB_TYPE_AUDIOSCROBBLER_RADIO_SOURCE,
 	                       "shell", shell,
@@ -321,13 +272,13 @@ rb_audioscrobbler_radio_source_new (RBAudioscrobblerProfilePage *parent,
                                "username", username,
 	                       "session-key", session_key,
 	                       "station-url", station_url,
-			       "toolbar-path", toolbar_path,
+			       "toolbar-menu", toolbar_menu,
 	                       NULL);
 
 	g_object_unref (shell);
 	g_object_unref (plugin);
 	g_object_unref (db);
-	g_free (toolbar_path);
+	g_object_unref (toolbar_menu);
 
 	return source;
 }
@@ -349,8 +300,9 @@ rb_audioscrobbler_radio_source_class_init (RBAudioscrobblerRadioSourceClass *kla
 	page_class = RB_DISPLAY_PAGE_CLASS (klass);
 	page_class->selected = impl_selected;
 	page_class->get_status = impl_get_status;
-	page_class->show_popup = impl_show_popup;
 	page_class->delete_thyself = impl_delete_thyself;
+	page_class->can_remove = impl_can_remove;
+	page_class->remove = impl_remove;
 
 	source_class = RB_SOURCE_CLASS (klass);
 	source_class->impl_can_rename = (RBSourceFeatureFunc) rb_true_function;
@@ -360,6 +312,7 @@ rb_audioscrobbler_radio_source_class_init (RBAudioscrobblerRadioSourceClass *kla
 	source_class->impl_try_playlist = (RBSourceFeatureFunc) rb_false_function;
 	source_class->impl_get_entry_view = impl_get_entry_view;
 	source_class->impl_handle_eos = impl_handle_eos;
+	source_class->impl_get_playback_status = impl_get_playback_status;
 
 	g_object_class_install_property (object_class,
 	                                 PROP_PARENT,
@@ -433,12 +386,8 @@ rb_audioscrobbler_radio_source_constructed (GObject *object)
 	RhythmDB *db;
 	GtkWidget *main_vbox;
 	GtkWidget *error_info_bar_content_area;
-	GtkWidget *password_info_bar_label;
-	GtkWidget *password_info_bar_content_area;
-	GObject *plugin;
-	GtkUIManager *ui_manager;
+	GtkAccelGroup *accel_group;
 	RBSourceToolbar *toolbar;
-	char *ui_file;
 
 	RB_CHAIN_GOBJECT_METHOD (rb_audioscrobbler_radio_source_parent_class, constructed, object);
 
@@ -447,7 +396,7 @@ rb_audioscrobbler_radio_source_constructed (GObject *object)
 	g_object_get (shell,
 		      "db", &db,
 		      "shell-player", &shell_player,
-		      "ui-manager", &ui_manager,
+		      "accel-group", &accel_group,
 		      NULL);
 
 	source->priv->art_store = rb_ext_db_new ("album-art");
@@ -457,7 +406,7 @@ rb_audioscrobbler_radio_source_constructed (GObject *object)
 	gtk_container_add (GTK_CONTAINER (source), main_vbox);
 
 	/* toolbar */
-	toolbar = rb_source_toolbar_new (RB_DISPLAY_PAGE (source), ui_manager);
+	toolbar = rb_source_toolbar_new (RB_DISPLAY_PAGE (source), accel_group);
 	gtk_box_pack_start (GTK_BOX (main_vbox), GTK_WIDGET (toolbar), FALSE, FALSE, 0);
 	gtk_widget_show_all (GTK_WIDGET (toolbar));
 
@@ -468,23 +417,6 @@ rb_audioscrobbler_radio_source_constructed (GObject *object)
 	gtk_container_add (GTK_CONTAINER (error_info_bar_content_area), source->priv->error_info_bar_label);
 	gtk_box_pack_start (GTK_BOX (main_vbox), source->priv->error_info_bar, FALSE, FALSE, 0);
 
-	/* password info bar */
-	source->priv->password_info_bar = gtk_info_bar_new ();
-	password_info_bar_label = gtk_label_new (_("You must enter your password to listen to this station"));
-	password_info_bar_content_area = gtk_info_bar_get_content_area (GTK_INFO_BAR (source->priv->password_info_bar));
-	gtk_container_add (GTK_CONTAINER (password_info_bar_content_area), password_info_bar_label);
-	source->priv->password_info_bar_entry = gtk_entry_new ();
-	gtk_entry_set_visibility (GTK_ENTRY (source->priv->password_info_bar_entry), FALSE);
-	gtk_info_bar_add_action_widget (GTK_INFO_BAR (source->priv->password_info_bar),
-	                                source->priv->password_info_bar_entry,
-	                                GTK_RESPONSE_NONE);
-	gtk_info_bar_add_button (GTK_INFO_BAR (source->priv->password_info_bar), GTK_STOCK_OK, GTK_RESPONSE_OK);
-	g_signal_connect (source->priv->password_info_bar,
-	                  "response",
-	                  G_CALLBACK (password_info_bar_response_cb),
-	                  source);
-	gtk_box_pack_start (GTK_BOX (main_vbox), source->priv->password_info_bar, FALSE, FALSE, 0);
-
 	/* entry view */
 	source->priv->track_view = rb_entry_view_new (db, G_OBJECT (shell_player), FALSE, FALSE);
 	rb_entry_view_append_column (source->priv->track_view, RB_ENTRY_VIEW_COL_TITLE, TRUE);
@@ -512,29 +444,12 @@ rb_audioscrobbler_radio_source_constructed (GObject *object)
 				 G_CALLBACK (playing_song_changed_cb),
 				 source, 0);
 
-	/* merge ui */
-	g_object_get (source, "plugin", &plugin, NULL);
-	ui_file = rb_find_plugin_data_file (plugin, "audioscrobbler-radio-ui.xml");
-	source->priv->ui_merge_id = gtk_ui_manager_add_ui_from_file (ui_manager, ui_file, NULL);
-
-	/* actions */
-	source->priv->action_group = _rb_display_page_register_action_group (RB_DISPLAY_PAGE (source),
-									     "AudioscrobblerRadioActions",
-									     NULL, 0,
-									     source);
-	_rb_action_group_add_display_page_actions (source->priv->action_group,
-						   G_OBJECT (shell),
-						   rb_audioscrobbler_radio_source_actions,
-						   G_N_ELEMENTS (rb_audioscrobbler_radio_source_actions));
-
 	rb_shell_append_display_page (shell, RB_DISPLAY_PAGE (source), RB_DISPLAY_PAGE (source->priv->parent));
 
 	g_object_unref (shell);
 	g_object_unref (shell_player);
 	g_object_unref (db);
-	g_object_unref (plugin);
-	g_object_unref (ui_manager);
-	g_free (ui_file);
+	g_object_unref (accel_group);
 }
 
 static void
@@ -580,11 +495,6 @@ rb_audioscrobbler_radio_source_finalize (GObject *object)
 	g_free (source->priv->session_key);
 	g_free (source->priv->station_url);
 
-	g_free (source->priv->old_api_password);
-	g_free (source->priv->old_api_session_id);
-	g_free (source->priv->old_api_base_url);
-	g_free (source->priv->old_api_base_path);
-
 	G_OBJECT_CLASS (rb_audioscrobbler_radio_source_parent_class)->finalize (object);
 }
 
@@ -730,7 +640,6 @@ tune (RBAudioscrobblerRadioSource *source)
 
 	source->priv->is_busy = TRUE;
 	gtk_widget_hide (source->priv->error_info_bar);
-	gtk_widget_hide (source->priv->password_info_bar);
 
 	sig_arg = g_strdup_printf ("api_key%smethodradio.tunesk%sstation%s%s",
 	                           rb_audioscrobbler_service_get_api_key (source->priv->service),
@@ -811,44 +720,38 @@ tune_response_cb (SoupSession *session,
 
 			rb_debug ("tune request responded with error: %s", message);
 
-			if (code == 4) {
-				/* Our API key only allows streaming of radio to subscribers */
-				rb_debug ("attempting to use old API to tune radio");
-				old_api_tune (source);
+			/* show appropriate error message */
+			char *error_message = NULL;
+
+			if (code == 6) {
+				/* Invalid station url */
+				error_message = g_strdup (_("Invalid station URL"));
+			} else if (code == 12) {
+				/* Subscriber only station */
+				/* Translators: %s is the name of the audioscrobbler service, for example "Last.fm".
+				 * This message indicates that to listen to this radio station the user needs to be
+				 * a paying subscriber to the service. */
+				error_message = g_strdup_printf (_("This station is only available to %s subscribers"),
+								 rb_audioscrobbler_service_get_name (source->priv->service));
+			} else if (code == 20) {
+				/* Not enough content */
+				error_message = g_strdup (_("Not enough content to play station"));
+			} else if (code == 27) {
+				/* Deprecated station */
+				/* Translators: %s is the name of the audioscrobbler service, for example "Last.fm".
+				 * This message indicates that the service has deprecated this type of station. */
+				error_message = g_strdup_printf (_("%s no longer supports this type of station"),
+								 rb_audioscrobbler_service_get_name (source->priv->service));
 			} else {
-				/* show appropriate error message */
-				char *error_message = NULL;
-
-				if (code == 6) {
-					/* Invalid station url */
-					error_message = g_strdup (_("Invalid station URL"));
-				} else if (code == 12) {
-					/* Subscriber only station */
-					/* Translators: %s is the name of the audioscrobbler service, for example "Last.fm".
-					 * This message indicates that to listen to this radio station the user needs to be
-					 * a paying subscriber to the service. */
-					error_message = g_strdup_printf (_("This station is only available to %s subscribers"),
-							                 rb_audioscrobbler_service_get_name (source->priv->service));
-				} else if (code == 20) {
-					/* Not enough content */
-					error_message = g_strdup (_("Not enough content to play station"));
-				} else if (code == 27) {
-					/* Deprecated station */
-					/* Translators: %s is the name of the audioscrobbler service, for example "Last.fm".
-					 * This message indicates that the service has deprecated this type of station. */
-					error_message = g_strdup_printf (_("%s no longer supports this type of station"),
-							                 rb_audioscrobbler_service_get_name (source->priv->service));
-				} else {
-					/* Other error */
-					error_message = g_strdup_printf (_("Error tuning station: %i - %s"), code, message);
-				}
-
-				display_error_info_bar (source, error_message);
-
-				g_free (error_message);
-
-				source->priv->is_busy = FALSE;
+				/* Other error */
+				error_message = g_strdup_printf (_("Error tuning station: %i - %s"), code, message);
 			}
+
+			display_error_info_bar (source, error_message);
+
+			g_free (error_message);
+
+			source->priv->is_busy = FALSE;
 		} else {
 			rb_debug ("unexpected response from tune request: %s", msg->response_body->data);
 			display_error_info_bar(source, _("Error tuning station: unexpected response"));
@@ -1068,229 +971,6 @@ xspf_entry_parsed (TotemPlParser *parser,
 	g_object_unref (db);
 }
 
-static void
-old_api_shake_hands (RBAudioscrobblerRadioSource *source)
-{
-	if (source->priv->old_api_password != NULL) {
-		char *password_hash;
-		char *msg_url;
-		SoupMessage *msg;
-
-		password_hash = g_compute_checksum_for_string (G_CHECKSUM_MD5, source->priv->old_api_password, -1);
-
-		msg_url = g_strdup_printf ("%sradio/handshake.php?username=%s&passwordmd5=%s",
-			                   rb_audioscrobbler_service_get_old_radio_api_url (source->priv->service),
-			                   source->priv->username,
-			                   password_hash);
-
-		rb_debug ("sending old api handshake request: %s", msg_url);
-		msg = soup_message_new ("GET", msg_url);
-		soup_session_queue_message (source->priv->soup_session,
-			                    msg,
-			                    old_api_handshake_response_cb,
-			                    source);
-
-		g_free (password_hash);
-		g_free (msg_url);
-	} else {
-#ifdef WITH_GNOME_KEYRING
-		GnomeKeyringResult result;
-		char *password;
-
-		rb_debug ("attempting to retrieve password from keyring");
-		result = gnome_keyring_find_password_sync (GNOME_KEYRING_NETWORK_PASSWORD,
-		                                           &password,
-		                                           "user", source->priv->username,
-		                                           "server", rb_audioscrobbler_service_get_name (source->priv->service),
-		                                           NULL);
-
-		if (result == GNOME_KEYRING_RESULT_OK) {
-			source->priv->old_api_password = g_strdup (password);
-			rb_debug ("password found. shaking hands");
-			old_api_shake_hands (source);
-		} else {
-			rb_debug ("no password found");
-#endif
-			rb_debug ("cannot shake hands. asking user for password");
-			display_password_info_bar (source);
-			source->priv->is_busy = FALSE;
-#ifdef WITH_GNOME_KEYRING
-		}
-#endif
-	}
-}
-
-static void
-old_api_handshake_response_cb (SoupSession *session,
-                               SoupMessage *msg,
-                               gpointer user_data)
-{
-	RBAudioscrobblerRadioSource *source;
-
-	source = RB_AUDIOSCROBBLER_RADIO_SOURCE (user_data);
-
-	if (msg->response_body->data == NULL) {
-		g_free (source->priv->old_api_session_id);
-		source->priv->old_api_session_id = NULL;
-		rb_debug ("handshake failed: no response");
-		display_error_info_bar (source, _("Error tuning station: no response"));
-	} else {
-		char **pieces;
-		int i;
-
-		pieces = g_strsplit (msg->response_body->data, "\n", 0);
-		for (i = 0; pieces[i] != NULL; i++) {
-			gchar **values = g_strsplit (pieces[i], "=", 2);
-
-			if (values[0] == NULL) {
-				rb_debug ("unexpected response content: %s", pieces[i]);
-			} else if (strcmp (values[0], "session") == 0) {
-				if (strcmp (values[1], "FAILED") == 0) {
-					g_free (source->priv->old_api_session_id);
-					source->priv->old_api_session_id = NULL;
-
-					rb_debug ("handshake failed: probably bad authentication. asking user for new password");
-					g_free (source->priv->old_api_password);
-					source->priv->old_api_password = NULL;
-					display_password_info_bar (source);
-				} else {
-					g_free (source->priv->old_api_session_id);
-					source->priv->old_api_session_id = g_strdup (values[1]);
-					rb_debug ("session ID: %s", source->priv->old_api_session_id);
-				}
-			} else if (strcmp (values[0], "base_url") == 0) {
-				g_free (source->priv->old_api_base_url);
-				source->priv->old_api_base_url = g_strdup (values[1]);
-				rb_debug ("base url: %s", source->priv->old_api_base_url);
-			} else if (strcmp (values[0], "base_path") == 0) {
-				g_free (source->priv->old_api_base_path);
-				source->priv->old_api_base_path = g_strdup (values[1]);
-				rb_debug ("base path: %s", source->priv->old_api_base_path);
-			} else if (strcmp (values[0], "banned") == 0) {
-				if (strcmp (values[1], "0") != 0) {
-					source->priv->old_api_is_banned = TRUE;
-				} else {
-					source->priv->old_api_is_banned = FALSE;
-				}
-				rb_debug ("banned: %i", source->priv->old_api_is_banned);
-			}
-
-			g_strfreev (values);
-		}
-		g_strfreev (pieces);
-	}
-
-	/* if handshake was successful then tune */
-	if (source->priv->old_api_session_id != NULL) {
-		old_api_tune (source);
-	} else {
-		source->priv->is_busy = FALSE;
-	}
-}
-
-static void
-old_api_tune (RBAudioscrobblerRadioSource *source)
-{
-	/* get a handshake first if we don't have one */
-	if (source->priv->old_api_session_id == NULL) {
-		old_api_shake_hands (source);
-	} else {
-		char *escaped_station_url;
-		char *msg_url;
-		SoupMessage *msg;
-
-		escaped_station_url = g_uri_escape_string (source->priv->station_url, NULL, FALSE);
-
-		msg_url = g_strdup_printf("http://%s%s/adjust.php?session=%s&url=%s",
-			                  source->priv->old_api_base_url,
-			                  source->priv->old_api_base_path,
-			                  source->priv->old_api_session_id,
-			                  escaped_station_url);
-
-		rb_debug ("sending old api tune request: %s", msg_url);
-		msg = soup_message_new ("GET", msg_url);
-		soup_session_queue_message (source->priv->soup_session,
-			                    msg,
-			                    old_api_tune_response_cb,
-			                    source);
-
-		g_free (escaped_station_url);
-		g_free (msg_url);
-	}
-}
-
-static void
-old_api_tune_response_cb (SoupSession *session,
-                          SoupMessage *msg,
-                          gpointer user_data)
-{
-	RBAudioscrobblerRadioSource *source;
-
-	source = RB_AUDIOSCROBBLER_RADIO_SOURCE (user_data);
-
-	if (msg->response_body->data != NULL) {
-		char **pieces;
-		int i;
-
-		pieces = g_strsplit (msg->response_body->data, "\n", 0);
-		for (i = 0; pieces[i] != NULL; i++) {
-			gchar **values = g_strsplit (pieces[i], "=", 2);
-
-			if (values[0] == NULL) {
-				rb_debug ("unexpected response from old api tune request: %s", pieces[i]);
-			} else if (strcmp (values[0], "response") == 0) {
-				if (strcmp (values[1], "OK") == 0) {
-					rb_debug ("old api tune request was successful");
-					/* no problems tuning, get the playlist */
-					old_api_fetch_playlist (source);
-				}
-			} else if (strcmp (values[0], "error") == 0) {
-				char *error_message;
-				rb_debug ("old api tune request responded with error: %s", pieces[i]);
-
-				error_message = g_strdup_printf (_("Error tuning station: %s"), values[1]);
-
-
-				g_free (error_message);
-
-				source->priv->is_busy = FALSE;
-			}
-			/* TODO: do something with other information given here */
-
-			g_strfreev (values);
-		}
-
-		g_strfreev (pieces);
-	} else {
-		rb_debug ("no response from old api tune request");
-		display_error_info_bar (source, _("Error tuning station: no response"));
-		source->priv->is_busy = FALSE;
-	}
-}
-
-static void
-old_api_fetch_playlist (RBAudioscrobblerRadioSource *source)
-{
-	char *msg_url;
-	SoupMessage *msg;
-
-	msg_url = g_strdup_printf("http://%s%s/xspf.php?sk=%s&discovery=%i&desktop=%s",
-		                  source->priv->old_api_base_url,
-		                  source->priv->old_api_base_path,
-		                  source->priv->old_api_session_id,
-		                  0,
-		                  "1.5");
-
-	rb_debug ("sending old api playlist request: %s", msg_url);
-	msg = soup_message_new ("GET", msg_url);
-	soup_session_queue_message (source->priv->soup_session,
-		                    msg,
-		                    fetch_playlist_response_cb,
-		                    source);
-
-	g_free (msg_url);
-}
-
 static void
 display_error_info_bar (RBAudioscrobblerRadioSource *source,
                         const char *message)
@@ -1300,65 +980,17 @@ display_error_info_bar (RBAudioscrobblerRadioSource *source,
 	gtk_widget_show_all (source->priv->error_info_bar);
 }
 
-static void
-display_password_info_bar (RBAudioscrobblerRadioSource *source)
+static gboolean
+impl_can_remove (RBDisplayPage *page)
 {
-	gtk_widget_show_all (source->priv->password_info_bar);
+	return TRUE;
 }
 
 static void
-password_info_bar_response_cb (GtkInfoBar *info_bar,
-                               int response_id,
-                               RBAudioscrobblerRadioSource *source)
+impl_remove (RBDisplayPage *page)
 {
-	gtk_widget_hide (source->priv->password_info_bar);
-
-	g_free (source->priv->old_api_password);
-	source->priv->old_api_password = g_strdup (gtk_entry_get_text (GTK_ENTRY (source->priv->password_info_bar_entry)));
-
-#ifdef WITH_GNOME_KEYRING
-	/* save the new password */
-	char *password_desc;
-
-	password_desc = g_strdup_printf (_("Password for streaming %s radio using the deprecated API"),
-	                                 rb_audioscrobbler_service_get_name (source->priv->service));
-
-	rb_debug ("saving password to keyring");
-	gnome_keyring_store_password_sync (GNOME_KEYRING_NETWORK_PASSWORD,
-	                                   GNOME_KEYRING_DEFAULT,
-	                                   password_desc,
-	                                   source->priv->old_api_password,
-	                                   "user", source->priv->username,
-	                                   "server", rb_audioscrobbler_service_get_name (source->priv->service),
-	                                   NULL);
-
-	g_free (password_desc);
-#endif
-
-	gtk_entry_set_text (GTK_ENTRY (source->priv->password_info_bar_entry), "");
-
-	old_api_shake_hands (source);
-}
-
-static void
-rename_station_action_cb (GtkAction *action, RBAudioscrobblerRadioSource *source)
-{
-	RBShell *shell;
-	RBDisplayPageTree *page_tree;
-
-	g_object_get (source, "shell", &shell, NULL);
-	g_object_get (shell, "display-page-tree", &page_tree, NULL);
-
-	rb_display_page_tree_edit_source_name (page_tree, RB_SOURCE (source));
-
-	g_object_unref (shell);
-	g_object_unref (page_tree);
-}
-
-static void
-delete_station_action_cb (GtkAction *action, RBAudioscrobblerRadioSource *source)
-{
-	rb_audioscrobbler_profile_page_remove_radio_station (source->priv->parent, RB_SOURCE (source));
+	RBAudioscrobblerRadioSource *source = RB_AUDIOSCROBBLER_RADIO_SOURCE (page);
+	rb_audioscrobbler_profile_page_remove_radio_station (source->priv->parent, RB_SOURCE (page));
 }
 
 static void
@@ -1386,38 +1018,32 @@ static void
 impl_get_status (RBDisplayPage *page, char **text, char **progress_text, float *progress)
 {
 	RBAudioscrobblerRadioSource *source = RB_AUDIOSCROBBLER_RADIO_SOURCE (page);
-
-	/* pulse progressbar if we're busy, otherwise see what the streaming source part of us has to say */
 	if (source->priv->is_busy) {
 		/* We could be calling either radio.tune or radio.getPlaylist methods.
 		 * "Tuning station" seems like a user friendly message to display for both cases.
 		 */
 		*progress_text = g_strdup (_("Tuning station"));
 		*progress = -1.0f;
-	} else {
-		rb_streaming_source_get_progress (RB_STREAMING_SOURCE (source), progress_text, progress);
 	}
 }
 
+static void
+impl_get_playback_status (RBSource *source, char **text, float *progress)
+{
+	rb_streaming_source_get_progress (RB_STREAMING_SOURCE (source), text, progress);
+}
+
 static RBSourceEOFType
 impl_handle_eos (RBSource *asource)
 {
 	return RB_SOURCE_EOF_NEXT;
 }
 
-static gboolean
-impl_show_popup (RBDisplayPage *page)
-{
-	_rb_display_page_show_popup (page, AUDIOSCROBBLER_RADIO_SOURCE_POPUP_PATH);
-	return TRUE;
-}
-
 static void
 impl_delete_thyself (RBDisplayPage *page)
 {
 	RBAudioscrobblerRadioSource *source;
 	RBShell *shell;
-	GtkUIManager *ui_manager;
 	RhythmDB *db;
 	GtkTreeIter iter;
 	gboolean loop;
@@ -1426,12 +1052,9 @@ impl_delete_thyself (RBDisplayPage *page)
 
 	source = RB_AUDIOSCROBBLER_RADIO_SOURCE (page);
 
-	g_object_get (source, "shell", &shell, "ui-manager", &ui_manager, NULL);
+	g_object_get (source, "shell", &shell, NULL);
 	g_object_get (shell, "db", &db, NULL);
 
-	/* unmerge ui */
-	gtk_ui_manager_remove_ui (ui_manager, source->priv->ui_merge_id);
-
 	/* Ensure playing entry isn't deleted twice */
 	source->priv->playing_entry = NULL;
 
@@ -1450,7 +1073,6 @@ impl_delete_thyself (RBDisplayPage *page)
 	rhythmdb_commit (db);
 
 	g_object_unref (shell);
-	g_object_unref (ui_manager);
 	g_object_unref (db);
 }
 
