@@ -51,6 +51,7 @@
 #include "rb-builder-helpers.h"
 #include "rb-uri-dialog.h"
 #include "rb-display-page-group.h"
+#include "rb-application.h"
 
 #include "rb-daap-container-record.h"
 #include "rb-daap-record-factory.h"
@@ -92,8 +93,7 @@ struct _RBDaapPlugin
 	gboolean sharing;
 	gboolean shutdown;
 
-	GtkActionGroup *daap_action_group;
-	guint daap_ui_merge_id;
+	GSimpleAction *new_share_action;
 
 	DMAPMdnsBrowser *mdns_browser;
 
@@ -121,8 +121,7 @@ G_MODULE_EXPORT void peas_register_types (PeasObjectModule *module);
 
 static void rb_daap_plugin_init (RBDaapPlugin *plugin);
 
-static void rb_daap_plugin_cmd_disconnect (GtkAction *action, RBSource *source);
-static void rb_daap_plugin_cmd_connect (GtkAction *action, RBDaapPlugin *plugin);
+static void new_share_action_cb (GSimpleAction *, GVariant *, gpointer);
 
 static void create_pixbufs (RBDaapPlugin *plugin);
 static void start_browsing (RBDaapPlugin *plugin);
@@ -149,20 +148,6 @@ RB_DEFINE_PLUGIN(RB_TYPE_DAAP_PLUGIN,
 		 (G_IMPLEMENT_INTERFACE_DYNAMIC (PEAS_GTK_TYPE_CONFIGURABLE,
 						peas_gtk_configurable_iface_init)))
 
-static GtkActionEntry rb_daap_plugin_actions [] =
-{
-	{ "MusicNewDAAPShare", GTK_STOCK_CONNECT, N_("Connect to _DAAP share..."), NULL,
-	  N_("Connect to a new DAAP share"),
-	  G_CALLBACK (rb_daap_plugin_cmd_connect) },
-};
-
-static GtkActionEntry rb_daap_source_actions[] =
-{
-	{ "DaapSourceDisconnect", GTK_STOCK_DISCONNECT, N_("_Disconnect"), NULL,
-	  N_("Disconnect from DAAP share"),
-	  G_CALLBACK (rb_daap_plugin_cmd_disconnect) },
-};
-
 static void
 rb_daap_plugin_init (RBDaapPlugin *plugin)
 {
@@ -176,6 +161,8 @@ rb_daap_plugin_init (RBDaapPlugin *plugin)
 	daap_settings = g_settings_new ("org.gnome.rhythmbox.plugins.daap");
 	plugin->dacp_settings = g_settings_get_child (daap_settings, "dacp");
 	g_object_unref (daap_settings);
+
+	rb_register_gst_plugin ();
 }
 
 static void
@@ -183,9 +170,8 @@ impl_activate (PeasActivatable *bplugin)
 {
 	RBDaapPlugin *plugin = RB_DAAP_PLUGIN (bplugin);
 	gboolean no_registration;
-	GtkUIManager *uimanager = NULL;
-	char *uifile;
 	RBShell *shell;
+	GApplication *app;
 
 	plugin->shutdown = FALSE;
 
@@ -206,29 +192,15 @@ impl_activate (PeasActivatable *bplugin)
 
 	create_pixbufs (plugin);
 
-	g_object_get (shell, "ui-manager", &uimanager, NULL);
+	app = g_application_get_default ();
+	plugin->new_share_action = g_simple_action_new ("daap-new-share", NULL);
+	g_signal_connect (plugin->new_share_action, "activate", G_CALLBACK (new_share_action_cb), plugin);
+	g_action_map_add_action (G_ACTION_MAP (app), G_ACTION (plugin->new_share_action));
 
-	/* add actions */
-	plugin->daap_action_group = gtk_action_group_new ("DaapActions");
-	gtk_action_group_set_translation_domain (plugin->daap_action_group,
-						 GETTEXT_PACKAGE);
-	gtk_action_group_add_actions (plugin->daap_action_group,
-				      rb_daap_plugin_actions, G_N_ELEMENTS (rb_daap_plugin_actions),
-				      plugin);
-	_rb_action_group_add_display_page_actions (plugin->daap_action_group,
-						   G_OBJECT (shell),
-						   rb_daap_source_actions,
-						   G_N_ELEMENTS (rb_daap_source_actions));
-	gtk_ui_manager_insert_action_group (uimanager, plugin->daap_action_group, 0);
-
-	/* add UI */
-	uifile = rb_find_plugin_data_file (G_OBJECT (plugin), "daap-ui.xml");
-	if (uifile != NULL) {
-		plugin->daap_ui_merge_id = gtk_ui_manager_add_ui_from_file (uimanager, uifile, NULL);
-		g_free (uifile);
-	}
-
-	g_object_unref (uimanager);
+	rb_application_add_plugin_menu_item (RB_APPLICATION (app),
+					     "display-page-add",
+					     "daap-new-share",
+					     g_menu_item_new (_("Connect to DAAP share..."), "app.daap-new-share"));
 
 	/*
 	 * Don't use daap when the no-registration flag is set.
@@ -254,7 +226,6 @@ static void
 impl_deactivate	(PeasActivatable *bplugin)
 {
 	RBDaapPlugin *plugin = RB_DAAP_PLUGIN (bplugin);
-	GtkUIManager *uimanager = NULL;
 	RBShell *shell;
 
 	rb_debug ("Shutting down DAAP plugin");
@@ -264,6 +235,9 @@ impl_deactivate	(PeasActivatable *bplugin)
 	unregister_daap_dbus_iface (plugin);
 	plugin->shutdown = TRUE;
 
+	rb_application_remove_plugin_menu_item (RB_APPLICATION (g_application_get_default ()),
+						"display-page-add",
+						"daap-new-share");
 	if (plugin->sharing)
 		rb_daap_sharing_shutdown (shell);
 
@@ -278,13 +252,6 @@ impl_deactivate	(PeasActivatable *bplugin)
 
 	g_object_unref (plugin->dacp_share);
 
-	g_object_get (shell, "ui-manager", &uimanager, NULL);
-
-	gtk_ui_manager_remove_ui (uimanager, plugin->daap_ui_merge_id);
-	gtk_ui_manager_remove_action_group (uimanager, plugin->daap_action_group);
-
-	g_object_unref (uimanager);
-
 	if (plugin->daap_share_pixbuf != NULL) {
 		g_object_unref (plugin->daap_share_pixbuf);
 		plugin->daap_share_pixbuf = NULL;
@@ -620,19 +587,6 @@ libdmapsharing_debug (const char *domain,
 	}
 }
 
-/* daap share connect/disconnect commands */
-
-static void
-rb_daap_plugin_cmd_disconnect (GtkAction *action, RBSource *source)
-{
-	if (!RB_IS_DAAP_SOURCE (source)) {
-		g_warning ("got non-Daap source for Daap action");
-		return;
-	}
-
-	rb_daap_source_disconnect (RB_DAAP_SOURCE (source));
-}
-
 static void
 new_daap_share_location_added_cb (RBURIDialog *dialog,
 				  const char *location,
@@ -671,9 +625,9 @@ new_daap_share_response_cb (GtkDialog *dialog, int response, gpointer meh)
 }
 
 static void
-rb_daap_plugin_cmd_connect (GtkAction *action,
-			    RBDaapPlugin *plugin)
+new_share_action_cb (GSimpleAction *action, GVariant *parameter, gpointer data)
 {
+	RBDaapPlugin *plugin = RB_DAAP_PLUGIN (data);
 	GtkWidget *dialog;
 
 	dialog = rb_uri_dialog_new (_("New DAAP share"), _("Host:port of DAAP share:"));
@@ -684,7 +638,6 @@ rb_daap_plugin_cmd_connect (GtkAction *action,
 	g_signal_connect (dialog, "response", G_CALLBACK (new_daap_share_response_cb), NULL);
 }
 
-
 /* daap:// URI -> RBDAAPSource mapping */
 
 static gboolean
@@ -741,22 +694,32 @@ rb_daap_plugin_shutdown (RBDaapPlugin *plugin)
 
 static void
 forget_remotes_button_toggled_cb (GtkToggleButton *button,
-				  RBDaapPlugin *plugin)
+				  gpointer data)
 {
-	g_settings_reset (plugin->dacp_settings, "known-remotes");
+	GSettings *dacp_settings;
+	GSettings *daap_settings;
+
+	daap_settings = g_settings_new ("org.gnome.rhythmbox.plugins.daap");
+	dacp_settings = g_settings_get_child (daap_settings, "dacp");
+	g_settings_reset (dacp_settings, "known-remotes");
+
+	g_object_unref (dacp_settings);
+	g_object_unref (daap_settings);
 }
 
 static gboolean
 share_name_entry_focus_out_event_cb (GtkEntry *entry,
 				     GdkEventFocus *event,
-				     RBDaapPlugin *plugin)
+				     gpointer data)
 {
+	GSettings  *settings;
 	gboolean    changed;
 	const char *name;
 	char       *old_name;
 
+	settings = g_settings_new ("org.gnome.rhythmbox.sharing");
 	name = gtk_entry_get_text (entry);
-	old_name = g_settings_get_string (plugin->settings, "share-name");
+	old_name = g_settings_get_string (settings, "share-name");
 
 	if (name == NULL && old_name == NULL) {
 		changed = FALSE;
@@ -769,10 +732,11 @@ share_name_entry_focus_out_event_cb (GtkEntry *entry,
 	}
 
 	if (changed) {
-		g_settings_set_string (plugin->settings, "share-name", name);
+		g_settings_set_string (settings, "share-name", name);
 	}
 
 	g_free (old_name);
+	g_object_unref (settings);
 
 	return FALSE;
 }
@@ -782,12 +746,14 @@ share_password_entry_focus_out_event_cb (GtkEntry *entry,
 					 GdkEventFocus *event,
 					 RBDaapPlugin *plugin)
 {
+	GSettings  *settings;
 	gboolean    changed;
 	const char *pw;
 	char       *old_pw;
 
 	pw = gtk_entry_get_text (entry);
-	old_pw = g_settings_get_string (plugin->settings, "share-password");
+	settings = g_settings_new ("org.gnome.rhythmbox.sharing");
+	old_pw = g_settings_get_string (settings, "share-password");
 
 	if (pw == NULL && old_pw == NULL) {
 		changed = FALSE;
@@ -800,10 +766,11 @@ share_password_entry_focus_out_event_cb (GtkEntry *entry,
 	}
 
 	if (changed) {
-		g_settings_set_string (plugin->settings, "share-password", pw);
+		g_settings_set_string (settings, "share-password", pw);
 	}
 
 	g_free (old_pw);
+	g_object_unref (settings);
 
 	return FALSE;
 }
@@ -850,7 +817,7 @@ update_config_widget (RBDaapPlugin *plugin)
 	g_signal_connect (name_entry,
 			  "focus-out-event",
 			  G_CALLBACK (share_name_entry_focus_out_event_cb),
-			  plugin);
+			  NULL);
 
 	password = g_settings_get_string (plugin->settings, "share-password");
 	if (password != NULL) {
@@ -860,7 +827,7 @@ update_config_widget (RBDaapPlugin *plugin)
 	g_signal_connect (password_entry,
 			  "focus-out-event",
 			  G_CALLBACK (share_password_entry_focus_out_event_cb),
-			  plugin);
+			  NULL);
 
 	/*gtk_widget_set_sensitive (password_entry, require_password);*/
 }
