@@ -35,8 +35,8 @@
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
 
-#ifdef WITH_GNOME_KEYRING
-#include <gnome-keyring.h>
+#ifdef WITH_LIBSECRET
+#include <libsecret/secret.h>
 #endif
 
 #include "rhythmdb.h"
@@ -51,6 +51,8 @@
 #include "rb-daap-record-factory.h"
 #include "rb-rhythmdb-dmap-db-adapter.h"
 #include "rb-display-page.h"
+#include "rb-builder-helpers.h"
+#include "rb-application.h"
 
 #include "rb-daap-plugin.h"
 
@@ -61,6 +63,7 @@
 typedef struct _RhythmDBEntryType RBDAAPEntryType;
 typedef struct _RhythmDBEntryTypeClass RBDAAPEntryTypeClass;
 
+static void rb_daap_source_constructed (GObject *object);
 static void rb_daap_source_dispose (GObject *object);
 static void rb_daap_source_set_property  (GObject *object,
 					  guint prop_id,
@@ -72,8 +75,8 @@ static void rb_daap_source_get_property  (GObject *object,
 				 	  GParamSpec *pspec);
 
 static void rb_daap_source_selected (RBDisplayPage *page);
-static gboolean rb_daap_source_show_popup (RBDisplayPage *page);
 static void rb_daap_source_get_status (RBDisplayPage *page, char **text, char **progress_text, float *progress);
+static void disconnect_action_cb (GSimpleAction *, GVariant *, gpointer);
 
 static void rb_daap_entry_type_class_init (RBDAAPEntryTypeClass *klass);
 static void rb_daap_entry_type_init (RBDAAPEntryType *etype);
@@ -81,8 +84,6 @@ GType rb_daap_entry_type_get_type (void);
 
 struct RBDAAPSourcePrivate
 {
-	GtkActionGroup *action_group;
-
 	char *service_name;
 	char *host;
 	guint port;
@@ -171,6 +172,7 @@ rb_daap_source_class_init (RBDAAPSourceClass *klass)
 	RBSourceClass *source_class = RB_SOURCE_CLASS (klass);
 	RBBrowserSourceClass *browser_source_class = RB_BROWSER_SOURCE_CLASS (klass);
 
+	object_class->constructed  = rb_daap_source_constructed;
 	object_class->dispose      = rb_daap_source_dispose;
 	object_class->finalize     = rb_daap_source_finalize;
 	object_class->get_property = rb_daap_source_get_property;
@@ -178,7 +180,6 @@ rb_daap_source_class_init (RBDAAPSourceClass *klass)
 
 	page_class->selected = rb_daap_source_selected;
 	page_class->get_status = rb_daap_source_get_status;
-	page_class->show_popup = rb_daap_source_show_popup;
 
 	source_class->impl_can_cut = (RBSourceFeatureFunc) rb_false_function;
 	source_class->impl_can_copy = (RBSourceFeatureFunc) rb_true_function;
@@ -294,6 +295,24 @@ rb_daap_source_get_property (GObject *object,
 	}
 }
 
+static void
+rb_daap_source_constructed (GObject *object)
+{
+	RBShell *shell;
+	GActionEntry actions[] = {
+		{ "daap-disconnect", disconnect_action_cb },
+	};
+
+	RB_CHAIN_GOBJECT_METHOD (rb_daap_source_parent_class, constructed, object);
+
+	g_object_get (object, "shell", &shell, NULL);
+	_rb_add_display_page_actions (G_ACTION_MAP (g_application_get_default ()),
+				      G_OBJECT (shell),
+				      actions,
+				      G_N_ELEMENTS (actions));
+	g_object_unref (shell);
+}
+
 
 RBSource *
 rb_daap_source_new (RBShell *shell,
@@ -310,6 +329,8 @@ rb_daap_source_new (RBShell *shell,
 	RhythmDB *db;
 	char *entry_type_name;
 	GSettings *settings;
+	GtkBuilder *builder;
+	GMenu *toolbar;
 
 	g_object_get (shell, "db", &db, NULL);
 	entry_type_name = g_strdup_printf ("daap:%s:%s:%s", service_name, name, host);
@@ -326,6 +347,10 @@ rb_daap_source_new (RBShell *shell,
 
 	icon = rb_daap_plugin_get_icon (RB_DAAP_PLUGIN (plugin), password_protected, FALSE);
 
+	builder = rb_builder_load_plugin_file (plugin, "daap-toolbar.ui", NULL);
+	toolbar = G_MENU (gtk_builder_get_object (builder, "daap-toolbar"));
+	rb_application_link_shared_menus (RB_APPLICATION (g_application_get_default ()), toolbar);
+
 	settings = g_settings_new ("org.gnome.rhythmbox.plugins.daap");
 	source = RB_SOURCE (g_object_new (RB_TYPE_DAAP_SOURCE,
 					  "service-name", service_name,
@@ -340,16 +365,16 @@ rb_daap_source_new (RBShell *shell,
 					  "plugin", G_OBJECT (plugin),
 					  "load-status", RB_SOURCE_LOAD_STATUS_NOT_LOADED,
 					  "settings", g_settings_get_child (settings, "source"),
-					  "toolbar-path", "/DAAPSourceToolBar",
+					  "toolbar-menu", toolbar,
 					  NULL));
 	g_object_unref (settings);
+	g_object_unref (builder);
 
 	if (icon != NULL) {
 		g_object_unref (icon);
 	}
 
-	rb_shell_register_entry_type_for_source (shell, source,
-						 entry_type);
+	rb_shell_register_entry_type_for_source (shell, source, entry_type);
 
 	return source;
 }
@@ -369,31 +394,36 @@ mount_op_reply_cb (GMountOperation *op,
 		   AuthData *auth_data)
 {
 	const char *password;
-	gchar *keyring;
-#ifdef WITH_GNOME_KEYRING
-	guint32 item_id;
+#ifdef WITH_LIBSECRET
+	gchar *label;
+	gchar *collection = NULL;
 #endif
 
 	rb_debug ("mount op reply: %d", result);
 	password = g_mount_operation_get_password (op);
 
-#ifdef WITH_GNOME_KEYRING
+#ifdef WITH_LIBSECRET
 	switch (g_mount_operation_get_password_save (op)) {
 	case G_PASSWORD_SAVE_NEVER:
 		break;
 
 	case G_PASSWORD_SAVE_FOR_SESSION:
-		keyring = "session";
+		collection = SECRET_COLLECTION_SESSION;
 		/* fall through */
 
 	case G_PASSWORD_SAVE_PERMANENTLY:
-		gnome_keyring_set_network_password_sync (keyring,
-			NULL,
-			"DAAP", auth_data->name,
-			NULL, "daap",
-			NULL, 0,
+		label = g_strdup_printf ("Rhythmbox DAAP password for %s", auth_data->name);
+		secret_password_store_sync (SECRET_SCHEMA_COMPAT_NETWORK,
+			collection,
+			label,
 			password,
-			&item_id);
+			NULL,
+			NULL,
+			"domain", "DAAP",
+			"server", auth_data->name,
+			"protocol", "daap",
+			NULL);
+		g_free (label);
 		break;
 
 	default:
@@ -418,7 +448,11 @@ mount_op_reply_cb (GMountOperation *op,
 }
 
 static void
-ask_password (RBDAAPSource *source, const char *name, const char *keyring, SoupSession *session, SoupMessage *msg, SoupAuth *auth)
+ask_password (RBDAAPSource *source,
+	      const char *name,
+	      SoupSession *session,
+	      SoupMessage *msg,
+	      SoupAuth *auth)
 {
 	GtkWindow *parent;
 	GMountOperation *mount_op;
@@ -440,10 +474,8 @@ ask_password (RBDAAPSource *source, const char *name, const char *keyring, SoupS
 	g_signal_connect (mount_op, "reply", G_CALLBACK (mount_op_reply_cb), auth_data);
 
 	flags = G_ASK_PASSWORD_NEED_PASSWORD;
-#ifdef WITH_GNOME_KEYRING
-	if (gnome_keyring_is_available ()) {
-		flags |= G_ASK_PASSWORD_SAVING_SUPPORTED;
-	}
+#ifdef WITH_LIBSECRET
+	flags |= G_ASK_PASSWORD_SAVING_SUPPORTED;
 #endif
 	message = g_strdup_printf (_("The music share '%s' requires a password to connect"), name);
 	g_signal_emit_by_name (mount_op, "ask-password", message, NULL, "DAAP", flags);
@@ -460,44 +492,31 @@ connection_auth_cb (DMAPConnection *connection,
 		    RBDAAPSource   *source)
 {
 	gchar *password = NULL;
-#ifdef WITH_GNOME_KEYRING
-	GnomeKeyringResult keyringret;
-	gchar *keyring;
-	GList *list = NULL;
+#ifdef WITH_LIBSECRET
+	GError *error = NULL;
 
-	keyring = NULL;
 	if (!source->priv->tried_password) {
-		gnome_keyring_get_default_keyring_sync (&keyring);
-		keyringret = gnome_keyring_find_network_password_sync (
-				NULL,
-				"DAAP", name,
-				NULL, "daap",
-				NULL, 0, &list);
-	} else {
-		keyringret = GNOME_KEYRING_RESULT_CANCELLED;
+		password = secret_password_lookup_sync (SECRET_SCHEMA_COMPAT_NETWORK,
+				NULL, &error,
+				"domain", "DAAP",
+				"server", name,
+				"protocol", "daap",
+				NULL);
 	}
 
-	if (keyringret == GNOME_KEYRING_RESULT_OK) {
-		GnomeKeyringNetworkPasswordData *pwd_data;
-
-		if (list != NULL) {
-			pwd_data = (GnomeKeyringNetworkPasswordData*)list->data;
-			password = g_strdup (pwd_data->password);
-		}
+	if (!error)
 		source->priv->tried_password = TRUE;
-	}
+	else
+		g_error_free (error);
 
 	if (password == NULL) {
-		ask_password (source, name, keyring, session, msg, auth);
+		ask_password (source, name, session, msg, auth);
 	} else {
 		dmap_connection_authenticate_message (connection, session, msg, auth, password);
 	}
 
-	if (list)
-		gnome_keyring_network_password_list_free (list);
-	g_free (keyring);
 #else
-	ask_password (source, name, NULL, session, msg, auth);
+	ask_password (source, name, session, msg, auth);
 #endif
 }
 
@@ -792,11 +811,11 @@ rb_daap_source_disconnect (RBDAAPSource *daap_source)
 	rb_debug ("DAAP connection finished");
 }
 
-static gboolean
-rb_daap_source_show_popup (RBDisplayPage *page)
+static void
+disconnect_action_cb (GSimpleAction *action, GVariant *parameter, gpointer data)
 {
-	_rb_display_page_show_popup (page, "/DAAPSourcePopup");
-	return TRUE;
+	RBDAAPSource *source = RB_DAAP_SOURCE (data);
+	rb_daap_source_disconnect (source);
 }
 
 SoupMessageHeaders *
