@@ -64,10 +64,7 @@ typedef struct
 {
 	PeasExtensionBase parent;
 
-	guint ui_merge_id;
-
 	GList *player_sources;
-	GtkActionGroup *actions;
 } RBGenericPlayerPlugin;
 
 typedef struct
@@ -80,24 +77,8 @@ G_MODULE_EXPORT void peas_register_types (PeasObjectModule  *module);
 
 static void rb_generic_player_plugin_init (RBGenericPlayerPlugin *plugin);
 
-static void rb_generic_player_plugin_new_playlist (GtkAction *action, RBSource *source);
-static void rb_generic_player_plugin_delete_playlist (GtkAction *action, RBSource *source);
-static void rb_generic_player_plugin_properties (GtkAction *action, RBSource *source);
-
 RB_DEFINE_PLUGIN(RB_TYPE_GENERIC_PLAYER_PLUGIN, RBGenericPlayerPlugin, rb_generic_player_plugin,)
 
-static GtkActionEntry rb_generic_player_plugin_actions[] = {
-	{ "GenericPlayerSourceNewPlaylist", RB_STOCK_PLAYLIST_NEW, N_("New Playlist"), NULL,
-	  N_("Create a new playlist on this device"),
-	  G_CALLBACK (rb_generic_player_plugin_new_playlist) },
-	{ "GenericPlayerPlaylistDelete", GTK_STOCK_DELETE, N_("Delete Playlist"), NULL,
-	  N_("Delete this playlist"),
-	  G_CALLBACK (rb_generic_player_plugin_delete_playlist) },
-	{ "GenericPlayerSourceProperties", GTK_STOCK_PROPERTIES, N_("_Properties"), NULL,
-	  N_("Display device properties"),
-	  G_CALLBACK (rb_generic_player_plugin_properties) }
-};
-
 static void
 rb_generic_player_plugin_init (RBGenericPlayerPlugin *plugin)
 {
@@ -110,43 +91,6 @@ rb_generic_player_plugin_source_deleted (RBGenericPlayerSource *source, RBGeneri
 	plugin->player_sources = g_list_remove (plugin->player_sources, source);
 }
 
-static void
-rb_generic_player_plugin_new_playlist (GtkAction *action, RBSource *source)
-{
-	RBShell *shell;
-	RBSource *playlist;
-	RBDisplayPageTree *page_tree;
-	RhythmDBEntryType *entry_type;
-
-	g_return_if_fail (RB_IS_GENERIC_PLAYER_SOURCE (source));
-	g_object_get (source,
-		      "shell", &shell,
-		      "entry-type", &entry_type,
-		      NULL);
-
-	playlist = rb_generic_player_playlist_source_new (shell, RB_GENERIC_PLAYER_SOURCE (source), NULL, NULL, entry_type);
-	g_object_unref (entry_type);
-
-	rb_generic_player_source_add_playlist (RB_GENERIC_PLAYER_SOURCE (source),
-					       shell,
-					       playlist);
-
-	g_object_get (shell, "display-page-tree", &page_tree, NULL);
-	rb_display_page_tree_edit_source_name (page_tree, playlist);
-	g_object_unref (page_tree);
-
-	g_object_unref (shell);
-}
-
-static void
-rb_generic_player_plugin_delete_playlist (GtkAction *action, RBSource *source)
-{
-	g_return_if_fail (RB_IS_GENERIC_PLAYER_PLAYLIST_SOURCE (source));
-
-	rb_generic_player_playlist_delete_from_player (RB_GENERIC_PLAYER_PLAYLIST_SOURCE (source));
-	rb_display_page_delete_thyself (RB_DISPLAY_PAGE (source));
-}
-
 static RBSource *
 create_source_cb (RBRemovableMediaManager *rmm, GMount *mount, MPIDDevice *device_info, RBGenericPlayerPlugin *plugin)
 {
@@ -162,33 +106,7 @@ create_source_cb (RBRemovableMediaManager *rmm, GMount *mount, MPIDDevice *devic
 	if (source == NULL && rb_generic_player_is_mount_player (mount, device_info))
 		source = rb_generic_player_source_new (G_OBJECT (plugin), shell, mount, device_info);
 
-	if (plugin->actions == NULL) {
-		plugin->actions = gtk_action_group_new ("GenericPlayerActions");
-		gtk_action_group_set_translation_domain (plugin->actions, GETTEXT_PACKAGE);
-
-		_rb_action_group_add_display_page_actions (plugin->actions,
-							   G_OBJECT (shell),
-							   rb_generic_player_plugin_actions,
-							   G_N_ELEMENTS (rb_generic_player_plugin_actions));
-	}
-
 	if (source) {
-		if (plugin->ui_merge_id == 0) {
-			GtkUIManager *uimanager = NULL;
-			char *file = NULL;
-
-			g_object_get (shell, "ui-manager", &uimanager, NULL);
-
-			gtk_ui_manager_insert_action_group (uimanager, plugin->actions, 0);
-
-			file = rb_find_plugin_data_file (G_OBJECT (plugin), "generic-player-ui.xml");
-			plugin->ui_merge_id = gtk_ui_manager_add_ui_from_file (uimanager,
-									       file,
-									       NULL);
-			g_free (file);
-			g_object_unref (uimanager);
-		}
-
 		plugin->player_sources = g_list_prepend (plugin->player_sources, source);
 		g_signal_connect_object (G_OBJECT (source),
 					 "deleted", G_CALLBACK (rb_generic_player_plugin_source_deleted),
@@ -233,13 +151,11 @@ impl_deactivate	(PeasActivatable *bplugin)
 {
 	RBGenericPlayerPlugin *plugin = RB_GENERIC_PLAYER_PLUGIN (bplugin);
 	RBRemovableMediaManager *rmm;
-	GtkUIManager *uimanager;
 	RBShell *shell;
 
 	g_object_get (plugin, "object", &shell, NULL);
 	g_object_get (shell,
 		      "removable-media-manager", &rmm,
-		      "ui-manager", &uimanager,
 		      NULL);
 
 	g_signal_handlers_disconnect_by_func (G_OBJECT (rmm), create_source_cb, plugin);
@@ -248,23 +164,10 @@ impl_deactivate	(PeasActivatable *bplugin)
 	g_list_free (plugin->player_sources);
 	plugin->player_sources = NULL;
 
-	if (plugin->ui_merge_id) {
-		gtk_ui_manager_remove_ui (uimanager, plugin->ui_merge_id);
-		plugin->ui_merge_id = 0;
-	}
-
-	g_object_unref (uimanager);
 	g_object_unref (rmm);
 	g_object_unref (shell);
 }
 
-static void
-rb_generic_player_plugin_properties (GtkAction *action, RBSource *source)
-{
-	g_return_if_fail (RB_IS_GENERIC_PLAYER_SOURCE (source));
-	rb_media_player_source_show_properties (RB_MEDIA_PLAYER_SOURCE (source));
-}
-
 G_MODULE_EXPORT void
 peas_register_types (PeasObjectModule *module)
 {
