@@ -462,6 +462,9 @@ rb_podcast_manager_download_entry (RBPodcastManager *pd,
 
 	if (entry == NULL)
 		return;
+	
+	if (rhythmdb_entry_get_boolean (entry, RHYTHMDB_PROP_HIDDEN))
+		return;
 
 	status = rhythmdb_entry_get_ulong (entry, RHYTHMDB_PROP_STATUS);
 	if ((status < RHYTHMDB_PODCAST_STATUS_COMPLETE) ||
@@ -611,7 +614,7 @@ rb_podcast_manager_head_query_cb (GtkTreeModel *query_model,
         uri = get_remote_location (entry);
 	status = rhythmdb_entry_get_ulong (entry, RHYTHMDB_PROP_STATUS);
 
-	if (status == 1)
+	if (status == RHYTHMDB_PODCAST_FEED_STATUS_NORMAL)
 		rb_podcast_manager_subscribe_feed (manager, uri, TRUE);
 
 	rhythmdb_entry_unref (entry);
@@ -1005,6 +1008,7 @@ gboolean
 rb_podcast_manager_subscribe_feed (RBPodcastManager *pd, const char *url, gboolean automatic)
 {
 	RBPodcastThreadInfo *info;
+	RhythmDBEntry *entry;
 	GFile *feed;
 	char *feed_url;
 	gboolean existing_feed;
@@ -1029,8 +1033,9 @@ rb_podcast_manager_subscribe_feed (RBPodcastManager *pd, const char *url, gboole
 #endif
 
 	feed_url = g_file_get_uri (feed);		/* not sure this buys us anything at all */
-	RhythmDBEntry *entry = rhythmdb_entry_lookup_by_location (pd->priv->db, feed_url);
+	entry = rhythmdb_entry_lookup_by_location (pd->priv->db, feed_url);
 	if (entry) {
+		GValue v = {0,};
 		if (rhythmdb_entry_get_entry_type (entry) != RHYTHMDB_ENTRY_TYPE_PODCAST_FEED) {
 			/* added as something else, probably iradio */
 			rb_error_dialog (NULL, _("URL already added"),
@@ -1039,6 +1044,12 @@ rb_podcast_manager_subscribe_feed (RBPodcastManager *pd, const char *url, gboole
 			return FALSE;
 		}
 		existing_feed = TRUE;
+
+		g_value_init (&v, G_TYPE_ULONG);
+		g_value_set_ulong (&v, RHYTHMDB_PODCAST_FEED_STATUS_UPDATING);
+		rhythmdb_entry_set (pd->priv->db, entry, RHYTHMDB_PROP_STATUS, &v);
+		rhythmdb_commit (pd->priv->db);
+		g_value_unset (&v);
 	} else {
 		existing_feed = FALSE;
 	}
@@ -1707,11 +1718,10 @@ rb_podcast_manager_unsubscribe_feed (RhythmDB *db, const char *url)
 	if (entry) {
 		GValue val = {0, };
 		g_value_init (&val, G_TYPE_ULONG);
-		g_value_set_ulong (&val, 0);
+		g_value_set_ulong (&val, RHYTHMDB_PODCAST_FEED_STATUS_HIDDEN);
 		rhythmdb_entry_set (db, entry, RHYTHMDB_PROP_STATUS, &val);
 		g_value_unset (&val);
 	}
-
 }
 
 gboolean
@@ -1877,6 +1887,10 @@ rb_podcast_manager_insert_feed_url (RBPodcastManager *pd, const char *url)
 	entry = rhythmdb_entry_lookup_by_location (pd->priv->db, url);
 	if (entry) {
 		rb_debug ("podcast feed entry for %s found", url);
+		g_value_init (&status_val, G_TYPE_ULONG);
+		g_value_set_ulong (&status_val, RHYTHMDB_PODCAST_FEED_STATUS_NORMAL);
+		rhythmdb_entry_set (pd->priv->db, entry, RHYTHMDB_PROP_STATUS, &status_val);
+		g_value_unset (&status_val);
 		return;
 	}
 	rb_debug ("adding podcast feed %s with no entries", url);
@@ -1887,7 +1901,7 @@ rb_podcast_manager_insert_feed_url (RBPodcastManager *pd, const char *url)
 		return;
 
 	g_value_init (&status_val, G_TYPE_ULONG);
-	g_value_set_ulong (&status_val, 1);
+	g_value_set_ulong (&status_val, RHYTHMDB_PODCAST_FEED_STATUS_NORMAL);
 	rhythmdb_entry_set (pd->priv->db, entry, RHYTHMDB_PROP_STATUS, &status_val);
 	g_value_unset (&status_val);
 
@@ -1924,9 +1938,14 @@ rb_podcast_manager_add_parsed_feed (RBPodcastManager *pd, RBPodcastChannel *data
 	gulong new_last_post;
 	const char *title;
 	GList *download_entries = NULL;
-	gboolean new_feed, updated, download_last;
+	gboolean new_feed, updated;
 	RhythmDB *db = pd->priv->db;
 	RhythmDBQueryModel *existing_entries = NULL;
+	enum {
+		DOWNLOAD_NONE,
+		DOWNLOAD_NEWEST,
+		DOWNLOAD_NEW
+	} download_mode;
 
 	RhythmDBEntry *entry;
 
@@ -1942,7 +1961,7 @@ rb_podcast_manager_add_parsed_feed (RBPodcastManager *pd, RBPodcastChannel *data
 
 		rb_debug ("Podcast feed entry for %s found", data->url);
 		g_value_init (&status_val, G_TYPE_ULONG);
-		g_value_set_ulong (&status_val, 1);
+		g_value_set_ulong (&status_val, RHYTHMDB_PODCAST_FEED_STATUS_NORMAL);
 		rhythmdb_entry_set (db, entry, RHYTHMDB_PROP_STATUS, &status_val);
 		g_value_unset (&status_val);
 		last_post = rhythmdb_entry_get_ulong (entry, RHYTHMDB_PROP_POST_TIME);
@@ -1970,7 +1989,7 @@ rb_podcast_manager_add_parsed_feed (RBPodcastManager *pd, RBPodcastChannel *data
 			return;
 
 		g_value_init (&status_val, G_TYPE_ULONG);
-		g_value_set_ulong (&status_val, 1);
+		g_value_set_ulong (&status_val, RHYTHMDB_PODCAST_FEED_STATUS_NORMAL);
 		rhythmdb_entry_set (db, entry, RHYTHMDB_PROP_STATUS, &status_val);
 		g_value_unset (&status_val);
 	}
@@ -2037,11 +2056,23 @@ rb_podcast_manager_add_parsed_feed (RBPodcastManager *pd, RBPodcastChannel *data
 	rhythmdb_entry_set (db, entry, RHYTHMDB_PROP_PLAYBACK_ERROR, &error_val);
 	g_value_unset (&error_val);
 
+	if (g_settings_get_enum (pd->priv->settings, PODCAST_DOWNLOAD_INTERVAL) == PODCAST_INTERVAL_MANUAL) {
+		/* if automatic updates are disabled, don't download anything */
+		rb_debug ("not downloading any new episodes");
+		download_mode = DOWNLOAD_NONE;
+	} else if (new_feed) {
+		/* don't download the entire backlog for new feeds */
+		rb_debug ("downloading most recent episodes");
+		download_mode = DOWNLOAD_NEWEST;
+	} else {
+		/* download all episodes since the last update for existing feeds */
+		rb_debug ("downloading all new episodes");
+		download_mode = DOWNLOAD_NEW;
+	}
+
 	/* insert episodes */
 	new_last_post = last_post;
-
 	updated = FALSE;
-	download_last = (g_settings_get_enum (pd->priv->settings, PODCAST_DOWNLOAD_INTERVAL) != PODCAST_INTERVAL_MANUAL);
 	for (lst_songs = data->posts; lst_songs != NULL; lst_songs = g_list_next (lst_songs)) {
 		RBPodcastItem *item = (RBPodcastItem *) lst_songs->data;
 		RhythmDBEntry *post_entry;
@@ -2088,16 +2119,23 @@ rb_podcast_manager_add_parsed_feed (RBPodcastManager *pd, RBPodcastChannel *data
 			updated = TRUE;
 
                 if (post_entry && item->pub_date >= new_last_post) {
-			if (item->pub_date > new_last_post) {
-				g_list_free (download_entries);
-				download_entries = NULL;
+			switch (download_mode) {
+			case DOWNLOAD_NEWEST:
+				if (item->pub_date > new_last_post) {
+					g_list_free (download_entries);
+					download_entries = NULL;
+				}
+				new_last_post = item->pub_date;
+				break;
+			case DOWNLOAD_NONE:
+			case DOWNLOAD_NEW:
+				break;
 			}
 			download_entries = g_list_prepend (download_entries, post_entry);
-			new_last_post = item->pub_date;
                 }
 	}
 
-	if (download_last) {
+	if (download_mode != DOWNLOAD_NONE) {
 		GValue status = {0,};
 		GList *t;
 
@@ -2175,6 +2213,11 @@ rb_podcast_manager_handle_feed_error (RBPodcastManager *mgr,
 		rhythmdb_entry_set (mgr->priv->db, entry, RHYTHMDB_PROP_PLAYBACK_ERROR, &v);
 		g_value_unset (&v);
 
+		g_value_init (&v, G_TYPE_ULONG);
+		g_value_set_ulong (&v, RHYTHMDB_PODCAST_FEED_STATUS_NORMAL);
+		rhythmdb_entry_set (mgr->priv->db, entry, RHYTHMDB_PROP_STATUS, &v);
+		g_value_unset (&v);
+
 		rhythmdb_commit (mgr->priv->db);
 		existing = TRUE;
 	}
