@@ -66,21 +66,17 @@
 #include "rb-cell-renderer-pixbuf.h"
 #include "rb-podcast-add-dialog.h"
 #include "rb-source-toolbar.h"
+#include "rb-builder-helpers.h"
+#include "rb-application.h"
+
+static void podcast_add_action_cb (GSimpleAction *, GVariant *, gpointer);
+static void podcast_download_action_cb (GSimpleAction *, GVariant *, gpointer);
+static void podcast_download_cancel_action_cb (GSimpleAction *, GVariant *, gpointer);
+static void podcast_feed_properties_action_cb (GSimpleAction *, GVariant *, gpointer);
+static void podcast_feed_update_action_cb (GSimpleAction *, GVariant *, gpointer);
+static void podcast_feed_update_all_action_cb (GSimpleAction *, GVariant *, gpointer);
+static void podcast_feed_delete_action_cb (GSimpleAction *, GVariant *, gpointer);
 
-static void podcast_cmd_new_podcast		(GtkAction *action,
-						 RBPodcastSource *source);
-static void podcast_cmd_download_post		(GtkAction *action,
-						 RBPodcastSource *source);
-static void podcast_cmd_cancel_download		(GtkAction *action,
-						 RBPodcastSource *source);
-static void podcast_cmd_delete_feed		(GtkAction *action,
-						 RBPodcastSource *source);
-static void podcast_cmd_update_feed		(GtkAction *action,
-						 RBPodcastSource *source);
-static void podcast_cmd_update_all		(GtkAction *action,
-						 RBPodcastSource *source);
-static void podcast_cmd_properties_feed		(GtkAction *action,
-						 RBPodcastSource *source);
 
 struct _RBPodcastSourcePrivate
 {
@@ -91,57 +87,30 @@ struct _RBPodcastSourcePrivate
 	GtkWidget *grid;
 	GtkWidget *paned;
 	GtkWidget *add_dialog;
-	GtkAction *add_action;
 	RBSourceToolbar *toolbar;
 
 	RhythmDBPropertyModel *feed_model;
 	RBPropertyView *feeds;
 	RBEntryView *posts;
-	GtkActionGroup *action_group;
 
 	GList *selected_feeds;
 	RhythmDBQuery *base_query;
 	RhythmDBQuery *search_query;
 	RBSourceSearch *default_search;
+	gboolean show_all_feeds;
 
 	RBPodcastManager *podcast_mgr;
 
 	GdkPixbuf *error_pixbuf;
+	GdkPixbuf *refresh_pixbuf;
+
+	GMenuModel *feed_popup;
+	GMenuModel *episode_popup;
+	GMenuModel *search_popup;
+	GAction *search_action;
 };
 
 
-static GtkActionEntry rb_podcast_source_actions [] =
-{
-	{ "MusicNewPodcast", RB_STOCK_PODCAST_NEW, N_("_New Podcast Feed..."), NULL,
-	  N_("Subscribe to a new podcast feed"),
-	  G_CALLBACK (podcast_cmd_new_podcast) },
-	{ "PodcastSrcDownloadPost", NULL, N_("Download _Episode"), NULL,
-	  N_("Download Podcast Episode"),
-	  G_CALLBACK (podcast_cmd_download_post) },
-	{ "PodcastSrcCancelDownload", GTK_STOCK_CANCEL, N_("_Cancel Download"), NULL,
-	  N_("Cancel Episode Download"),
-	  G_CALLBACK (podcast_cmd_cancel_download) },
-	{ "PodcastFeedProperties", GTK_STOCK_PROPERTIES, N_("_Properties"), NULL,
-	  N_("Episode Properties"),
-	  G_CALLBACK (podcast_cmd_properties_feed) },
-	{ "PodcastFeedUpdate", GTK_STOCK_REFRESH, N_("_Update Podcast Feed"), NULL,
-	  N_("Update Feed"),
-	  G_CALLBACK (podcast_cmd_update_feed) },
-	{ "PodcastFeedDelete", GTK_STOCK_DELETE, N_("_Delete Podcast Feed"), NULL,
-	  N_("Delete Feed"),
-	  G_CALLBACK (podcast_cmd_delete_feed) },
-	{ "PodcastUpdateAllFeeds", GTK_STOCK_REFRESH, N_("_Update All Feeds"), NULL,
-	  N_("Update all feeds"),
-	  G_CALLBACK (podcast_cmd_update_all) },
-};
-
-static GtkRadioActionEntry rb_podcast_source_radio_actions [] =
-{
-	{ "PodcastSearchAll", NULL, N_("Search all fields"), NULL, NULL, RHYTHMDB_PROP_SEARCH_MATCH },
-	{ "PodcastSearchFeeds", NULL, N_("Search podcast feeds"), NULL, NULL, RHYTHMDB_PROP_ALBUM_FOLDED },
-	{ "PodcastSearchEpisodes", NULL, N_("Search podcast episodes"), NULL, NULL, RHYTHMDB_PROP_TITLE_FOLDED }
-};
-
 static const GtkTargetEntry posts_view_drag_types[] = {
 	{  "text/uri-list", 0, 0 },
 	{  "_NETSCAPE_URL", 0, 1 },
@@ -153,6 +122,7 @@ enum
 	PROP_0,
 	PROP_PODCAST_MANAGER,
 	PROP_BASE_QUERY,
+	PROP_SHOW_ALL_FEEDS,
 	PROP_SHOW_BROWSER
 };
 
@@ -172,74 +142,68 @@ podcast_posts_show_popup_cb (RBEntryView *view,
 			     gboolean over_entry,
 			     RBPodcastSource *source)
 {
-	if (G_OBJECT (source) == NULL) {
-		return;
-	} else if (!over_entry) {
-		_rb_display_page_show_popup (RB_DISPLAY_PAGE (source), "/PodcastSourcePopup");
-	} else {
-		GtkAction* action;
-		GList *lst;
-		gboolean downloadable = FALSE;
-		gboolean cancellable = FALSE;
+	GAction* action;
+	GList *lst;
+	gboolean downloadable = FALSE;
+	gboolean cancellable = FALSE;
+	GtkWidget *menu;
+	GActionMap *map;
 
-		lst = rb_entry_view_get_selected_entries (view);
+	lst = rb_entry_view_get_selected_entries (view);
 
-		while (lst) {
-			RhythmDBEntry *entry = (RhythmDBEntry*) lst->data;
-			gulong status = rhythmdb_entry_get_ulong (entry, RHYTHMDB_PROP_STATUS);
+	while (lst) {
+		RhythmDBEntry *entry = (RhythmDBEntry*) lst->data;
+		gulong status = rhythmdb_entry_get_ulong (entry, RHYTHMDB_PROP_STATUS);
 
-			if (rb_podcast_manager_entry_in_download_queue (source->priv->podcast_mgr, entry)) {
-				cancellable = TRUE;
-			} else if (status != RHYTHMDB_PODCAST_STATUS_COMPLETE) {
-				downloadable = TRUE;
-			}
-
-			lst = lst->next;
+		if (rb_podcast_manager_entry_in_download_queue (source->priv->podcast_mgr, entry)) {
+			cancellable = TRUE;
+		} else if (status != RHYTHMDB_PODCAST_STATUS_COMPLETE) {
+			downloadable = TRUE;
 		}
 
-		g_list_foreach (lst, (GFunc)rhythmdb_entry_unref, NULL);
-		g_list_free (lst);
-
-		action = gtk_action_group_get_action (source->priv->action_group, "PodcastSrcDownloadPost");
-		gtk_action_set_sensitive (action, downloadable);
-
-		action = gtk_action_group_get_action (source->priv->action_group, "PodcastSrcCancelDownload");
-		gtk_action_set_sensitive (action, cancellable);
-
-		_rb_display_page_show_popup (RB_DISPLAY_PAGE (source), "/PodcastViewPopup");
+		lst = lst->next;
 	}
+
+	g_list_foreach (lst, (GFunc)rhythmdb_entry_unref, NULL);
+	g_list_free (lst);
+
+	map = G_ACTION_MAP (g_application_get_default ());
+	action = g_action_map_lookup_action (map, "podcast-download");
+	g_simple_action_set_enabled (G_SIMPLE_ACTION (action), downloadable);
+
+	action = g_action_map_lookup_action (map, "podcast-cancel-download");
+	g_simple_action_set_enabled (G_SIMPLE_ACTION (action), cancellable);
+
+	menu = gtk_menu_new_from_model (source->priv->episode_popup);
+	gtk_menu_attach_to_widget (GTK_MENU (menu), GTK_WIDGET (source), NULL);
+	gtk_menu_popup (GTK_MENU (menu), NULL, NULL, NULL, NULL, 3, gtk_get_current_event_time ());
 }
 
 static void
 podcast_feeds_show_popup_cb (RBPropertyView *view,
 			     RBPodcastSource *source)
 {
-	if (G_OBJECT (source) == NULL) {
-		return;
-	} else {
-		GtkAction *act_update;
-		GtkAction *act_properties;
-		GtkAction *act_delete;
-		GList *lst;
+	GActionMap *map;
+	GAction *act_update;
+	GAction *act_properties;
+	GAction *act_delete;
+	GtkWidget *menu;
+	GList *lst;
 
-		lst = source->priv->selected_feeds;
+	lst = source->priv->selected_feeds;
 
-		act_update = gtk_action_group_get_action (source->priv->action_group, "PodcastFeedUpdate");
-		act_properties = gtk_action_group_get_action (source->priv->action_group, "PodcastFeedProperties");
-		act_delete = gtk_action_group_get_action (source->priv->action_group, "PodcastFeedDelete");
+	map = G_ACTION_MAP (g_application_get_default ());
+	act_update = g_action_map_lookup_action (map, "podcast-feed-update");
+	act_properties = g_action_map_lookup_action (map, "podcast-feed-properties");
+	act_delete = g_action_map_lookup_action (map, "podcast-feed-delete");
 
-		if (lst) {
-			gtk_action_set_visible (act_update, TRUE);
-			gtk_action_set_visible (act_properties, TRUE);
-			gtk_action_set_visible (act_delete, TRUE);
-		} else {
-			gtk_action_set_visible (act_update, FALSE);
-			gtk_action_set_visible (act_properties, FALSE);
-			gtk_action_set_visible (act_delete, FALSE);
-		}
+	g_simple_action_set_enabled (G_SIMPLE_ACTION (act_update), lst != NULL);
+	g_simple_action_set_enabled (G_SIMPLE_ACTION (act_properties), lst != NULL);
+	g_simple_action_set_enabled (G_SIMPLE_ACTION (act_delete), lst != NULL);
 
-		_rb_display_page_show_popup (RB_DISPLAY_PAGE (source), "/PodcastFeedViewPopup");
-	}
+	menu = gtk_menu_new_from_model (source->priv->feed_popup);
+	gtk_menu_attach_to_widget (GTK_MENU (menu), GTK_WIDGET (source), NULL);
+	gtk_menu_popup (GTK_MENU (menu), NULL, NULL, NULL, NULL, 3, gtk_get_current_event_time ());
 }
 
 static GPtrArray *
@@ -289,7 +253,7 @@ construct_query_from_selection (RBPodcastSource *source)
 }
 
 static void
-rb_podcast_source_do_query (RBPodcastSource *source)
+rb_podcast_source_do_query (RBPodcastSource *source, gboolean feed_query)
 {
 	RhythmDBQueryModel *query_model;
 	GPtrArray *query;
@@ -300,6 +264,41 @@ rb_podcast_source_do_query (RBPodcastSource *source)
 	rb_entry_view_set_model (source->priv->posts, query_model);
 	g_object_set (source, "query-model", query_model, NULL);
 
+	if (feed_query) {
+		if (source->priv->feed_model != NULL) {
+			g_object_unref (source->priv->feed_model);
+			source->priv->feed_model = NULL;
+		}
+
+		if (source->priv->show_all_feeds && (source->priv->search_query == NULL)) {
+			RhythmDBQueryModel *feed_query_model;
+
+			rb_debug ("showing all feeds in browser");
+			source->priv->feed_model = rhythmdb_property_model_new (source->priv->db, RHYTHMDB_PROP_LOCATION);
+			g_object_set (source->priv->feeds, "property-model", source->priv->feed_model, NULL);
+
+			feed_query_model = rhythmdb_query_model_new_empty (source->priv->db);
+			g_object_set (source->priv->feed_model, "query-model", feed_query_model, NULL);
+
+			rhythmdb_do_full_query_async (source->priv->db,
+						      RHYTHMDB_QUERY_RESULTS (feed_query_model),
+						      RHYTHMDB_QUERY_PROP_EQUALS,
+						      RHYTHMDB_PROP_TYPE,
+						      RHYTHMDB_ENTRY_TYPE_PODCAST_FEED,
+						      RHYTHMDB_QUERY_PROP_NOT_EQUAL,
+						      RHYTHMDB_PROP_STATUS,
+						      RHYTHMDB_PODCAST_FEED_STATUS_HIDDEN,
+						      RHYTHMDB_QUERY_END);
+			g_object_unref (feed_query_model);
+		} else {
+			rb_debug ("only showing matching feeds in browser");
+			source->priv->feed_model = rhythmdb_property_model_new (source->priv->db, RHYTHMDB_PROP_SUBTITLE);
+			g_object_set (source->priv->feeds, "property-model", source->priv->feed_model, NULL);
+
+			g_object_set (source->priv->feed_model, "query-model", query_model, NULL);
+		}
+	}
+
 	/* build and run the query */
 	query = construct_query_from_selection (source);
 	rhythmdb_do_full_query_async_parsed (source->priv->db,
@@ -326,11 +325,10 @@ feed_select_change_cb (RBPropertyView *propview,
 
 	source->priv->selected_feeds = rb_string_list_copy (feeds);
 
-	rb_podcast_source_do_query (source);
+	rb_podcast_source_do_query (source, FALSE);
 	rb_source_notify_filter_changed (RB_SOURCE (source));
 }
 
-
 static void
 posts_view_drag_data_received_cb (GtkWidget *widget,
 				  GdkDragContext *dc,
@@ -347,7 +345,7 @@ posts_view_drag_data_received_cb (GtkWidget *widget,
 static void
 podcast_add_dialog_closed_cb (RBPodcastAddDialog *dialog, RBPodcastSource *source)
 {
-	rb_podcast_source_do_query (source);
+	rb_podcast_source_do_query (source, FALSE);
 	gtk_widget_set_margin_top (GTK_WIDGET (source->priv->grid), 6);
 	gtk_widget_hide (source->priv->add_dialog);
 	gtk_widget_show (GTK_WIDGET (source->priv->toolbar));
@@ -374,8 +372,9 @@ yank_clipboard_url (GtkClipboard *clipboard, const char *text, RBPodcastSource *
 }
 
 static void
-podcast_cmd_new_podcast (GtkAction *action, RBPodcastSource *source)
+podcast_add_action_cb (GSimpleAction *action, GVariant *parameter, gpointer data)
 {
+	RBPodcastSource *source = RB_PODCAST_SOURCE (data);
 	RhythmDBQueryModel *query_model;
 
 	rb_podcast_add_dialog_reset (RB_PODCAST_ADD_DIALOG (source->priv->add_dialog), NULL, FALSE);
@@ -404,14 +403,18 @@ podcast_cmd_new_podcast (GtkAction *action, RBPodcastSource *source)
 void
 rb_podcast_source_add_feed (RBPodcastSource *source, const char *text)
 {
-	gtk_action_activate (source->priv->add_action);
+	GtkWidget *window;
+
+	window = gtk_widget_get_toplevel (GTK_WIDGET (source));
+	g_action_group_activate_action (G_ACTION_GROUP (window), "podcast-add", NULL);
 
 	rb_podcast_add_dialog_reset (RB_PODCAST_ADD_DIALOG (source->priv->add_dialog), text, TRUE);
 }
 
 static void
-podcast_cmd_download_post (GtkAction *action, RBPodcastSource *source)
+podcast_download_action_cb (GSimpleAction *action, GVariant *parameter, gpointer data)
 {
+	RBPodcastSource *source = RB_PODCAST_SOURCE (data);
 	GList *lst;
 	GValue val = {0, };
 	RBEntryView *posts;
@@ -443,8 +446,9 @@ podcast_cmd_download_post (GtkAction *action, RBPodcastSource *source)
 }
 
 static void
-podcast_cmd_cancel_download (GtkAction *action, RBPodcastSource *source)
+podcast_download_cancel_action_cb (GSimpleAction *action, GVariant *parameter, gpointer data)
 {
+	RBPodcastSource *source = RB_PODCAST_SOURCE (data);
 	GList *lst;
 	GValue val = {0, };
 	RBEntryView *posts;
@@ -500,8 +504,9 @@ podcast_remove_response_cb (GtkDialog *dialog, int response, RBPodcastSource *so
 }
 
 static void
-podcast_cmd_delete_feed (GtkAction *action, RBPodcastSource *source)
+podcast_feed_delete_action_cb (GSimpleAction *action, GVariant *parameter, gpointer data)
 {
+	RBPodcastSource *source = RB_PODCAST_SOURCE (data);
 	GtkWidget *dialog;
 	GtkWidget *button;
 	GtkWindow *window;
@@ -546,8 +551,9 @@ podcast_cmd_delete_feed (GtkAction *action, RBPodcastSource *source)
 }
 
 static void
-podcast_cmd_properties_feed (GtkAction *action, RBPodcastSource *source)
+podcast_feed_properties_action_cb (GSimpleAction *action, GVariant *parameter, gpointer data)
 {
+	RBPodcastSource *source = RB_PODCAST_SOURCE (data);
 	RhythmDBEntry *entry;
 	GtkWidget *dialog;
 	const char *location;
@@ -568,8 +574,9 @@ podcast_cmd_properties_feed (GtkAction *action, RBPodcastSource *source)
 }
 
 static void
-podcast_cmd_update_feed (GtkAction *action, RBPodcastSource *source)
+podcast_feed_update_action_cb (GSimpleAction *action, GVariant *parameter, gpointer data)
 {
+	RBPodcastSource *source = RB_PODCAST_SOURCE (data);
 	GList *feeds, *l;
 
 	rb_debug ("Update action");
@@ -592,8 +599,9 @@ podcast_cmd_update_feed (GtkAction *action, RBPodcastSource *source)
 }
 
 static void
-podcast_cmd_update_all (GtkAction *action, RBPodcastSource *source)
+podcast_feed_update_all_action_cb (GSimpleAction *action, GVariant *parameter, gpointer data)
 {
+	RBPodcastSource *source = RB_PODCAST_SOURCE (data);
 	rb_podcast_manager_update_feeds (source->priv->podcast_mgr);
 }
 
@@ -744,11 +752,11 @@ podcast_feed_title_cell_data_func (GtkTreeViewColumn *column,
 }
 
 static void
-podcast_feed_error_cell_data_func (GtkTreeViewColumn *column,
-				   GtkCellRenderer *renderer,
-				   GtkTreeModel *tree_model,
-				   GtkTreeIter *iter,
-				   RBPodcastSource *source)
+podcast_feed_pixbuf_cell_data_func (GtkTreeViewColumn *column,
+				    GtkCellRenderer *renderer,
+				    GtkTreeModel *tree_model,
+				    GtkTreeIter *iter,
+				    RBPodcastSource *source)
 {
 	char *title;
 	RhythmDBEntry *entry = NULL;
@@ -762,7 +770,10 @@ podcast_feed_error_cell_data_func (GtkTreeViewColumn *column,
 	g_free (title);
 
 	if (entry != NULL) {
-		if (rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_PLAYBACK_ERROR)) {
+		gulong status = rhythmdb_entry_get_ulong (entry, RHYTHMDB_PROP_STATUS);
+		if (status == RHYTHMDB_PODCAST_FEED_STATUS_UPDATING) {
+			pixbuf = source->priv->refresh_pixbuf;
+		} else if (rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_PLAYBACK_ERROR)) {
 			pixbuf = source->priv->error_pixbuf;
 		}
 	}
@@ -913,9 +924,13 @@ podcast_entry_changed_cb (RhythmDB *db,
 		GValue *v = &g_array_index (changes, GValue, i);
 		RhythmDBEntryChange *change = g_value_get_boxed (v);
 
-		if (change->prop == RHYTHMDB_PROP_PLAYBACK_ERROR) {
+		switch (change->prop) {
+		case RHYTHMDB_PROP_PLAYBACK_ERROR:
+		case RHYTHMDB_PROP_STATUS:
 			feed_changed = TRUE;
 			break;
+		default:
+			break;
 		}
 	}
 
@@ -940,9 +955,9 @@ podcast_entry_changed_cb (RhythmDB *db,
 }
 
 static void
-podcast_error_pixbuf_clicked_cb (RBCellRendererPixbuf *renderer,
-				 const char *path_string,
-				 RBPodcastSource *source)
+podcast_status_pixbuf_clicked_cb (RBCellRendererPixbuf *renderer,
+				  const char *path_string,
+				  RBPodcastSource *source)
 {
 	GtkTreePath *path;
 	GtkTreeIter iter;
@@ -993,9 +1008,15 @@ rb_podcast_source_new (RBShell *shell,
 {
 	RBSource *source;
 	GSettings *settings;
+	GtkBuilder *builder;
+	GMenu *toolbar;
 
 	settings = g_settings_new (PODCAST_SETTINGS_SCHEMA);
 
+	builder = rb_builder_load ("podcast-toolbar.ui", NULL);
+	toolbar = G_MENU (gtk_builder_get_object (builder, "podcast-toolbar"));
+	rb_application_link_shared_menus (RB_APPLICATION (g_application_get_default ()), toolbar);
+
 	source = RB_SOURCE (g_object_new (RB_TYPE_PODCAST_SOURCE,
 					  "name", name,
 					  "shell", shell,
@@ -1003,9 +1024,10 @@ rb_podcast_source_new (RBShell *shell,
 					  "podcast-manager", podcast_manager,
 					  "base-query", base_query,
 					  "settings", g_settings_get_child (settings, "source"),
-					  "toolbar-path", "/PodcastSourceToolBar",
+					  "toolbar-menu", toolbar,
 					  NULL));
 	g_object_unref (settings);
+	g_object_unref (builder);
 
 	if (icon_name != NULL) {
 		GdkPixbuf *pixbuf;
@@ -1214,7 +1236,7 @@ impl_search (RBSource *asource, RBSourceSearch *search, const char *cur_text, co
 		source->priv->search_query = NULL;
 	}
 	source->priv->search_query = rb_source_search_create_query (search, source->priv->db, new_text);
-	rb_podcast_source_do_query (source);
+	rb_podcast_source_do_query (source, TRUE);
 
 	rb_source_notify_filter_changed (RB_SOURCE (source));
 }
@@ -1223,20 +1245,15 @@ static void
 impl_reset_filters (RBSource *asource)
 {
 	RBPodcastSource *source = RB_PODCAST_SOURCE (asource);
+
 	if (source->priv->search_query != NULL) {
 		rhythmdb_query_free (source->priv->search_query);
 		source->priv->search_query = NULL;
 	}
 	rb_source_toolbar_clear_search_entry (source->priv->toolbar);
-
+	
 	rb_property_view_set_selection (source->priv->feeds, NULL);
-}
-
-static gboolean
-impl_show_popup (RBDisplayPage *page)
-{
-	_rb_display_page_show_popup (page, "/PodcastSourcePopup");
-	return TRUE;
+	rb_podcast_source_do_query (source, TRUE);
 }
 
 static void
@@ -1274,9 +1291,9 @@ impl_get_status (RBDisplayPage *page, char **text, char **progress_text, float *
 
 
 static char *
-impl_get_delete_action (RBSource *source)
+impl_get_delete_label (RBSource *source)
 {
-	return g_strdup ("EditDelete");
+	return g_strdup (_("Delete"));
 }
 
 static void
@@ -1291,6 +1308,9 @@ impl_set_property (GObject *object, guint prop_id, const GValue *value, GParamSp
 	case PROP_BASE_QUERY:
 		source->priv->base_query = rhythmdb_query_copy (g_value_get_pointer (value));
 		break;
+	case PROP_SHOW_ALL_FEEDS:
+		source->priv->show_all_feeds = g_value_get_boolean (value);
+		break;
 	case PROP_SHOW_BROWSER:
 		gtk_widget_set_visible (GTK_WIDGET (source->priv->feeds), g_value_get_boolean (value));
 		break;
@@ -1312,6 +1332,9 @@ impl_get_property (GObject *object, guint prop_id, GValue *value, GParamSpec *ps
 	case PROP_BASE_QUERY:
 		g_value_set_pointer (value, source->priv->base_query);
 		break;
+	case PROP_SHOW_ALL_FEEDS:
+		g_value_set_boolean (value, source->priv->show_all_feeds);
+		break;
 	case PROP_SHOW_BROWSER:
 		g_value_set_boolean (value, gtk_widget_get_visible (GTK_WIDGET (source->priv->feeds)));
 		break;
@@ -1329,11 +1352,20 @@ impl_constructed (GObject *object)
 	GtkCellRenderer *renderer;
 	RBShell *shell;
 	RBShellPlayer *shell_player;
-	RhythmDBQueryModel *query_model;
-	GtkAction *action;
 	GSettings *settings;
 	int position;
-	GtkUIManager *ui_manager;
+	GtkAccelGroup *accel_group;
+	GtkBuilder *builder;
+	GMenu *section;
+	GActionEntry actions[] = {
+		{ "podcast-add", podcast_add_action_cb },
+		{ "podcast-download", podcast_download_action_cb },
+		{ "podcast-cancel-download", podcast_download_cancel_action_cb },
+		{ "podcast-feed-properties", podcast_feed_properties_action_cb },
+		{ "podcast-feed-update", podcast_feed_update_action_cb },
+		{ "podcast-feed-update-all", podcast_feed_update_all_action_cb },
+		{ "podcast-feed-delete", podcast_feed_delete_action_cb }
+	};
 
 	RB_CHAIN_GOBJECT_METHOD (rb_podcast_source_parent_class, constructed, object);
 	source = RB_PODCAST_SOURCE (object);
@@ -1342,45 +1374,17 @@ impl_constructed (GObject *object)
 	g_object_get (shell,
 		      "db", &source->priv->db,
 		      "shell-player", &shell_player,
-		      "ui-manager", &ui_manager,
+		      "accel-group", &accel_group,
 		      NULL);
 
-	source->priv->action_group = _rb_display_page_register_action_group (RB_DISPLAY_PAGE (source),
-									     "PodcastActions",
-									     NULL, 0,
-									     source);
+	_rb_add_display_page_actions (G_ACTION_MAP (g_application_get_default ()), G_OBJECT (shell), actions, G_N_ELEMENTS (actions));
 
-	_rb_action_group_add_display_page_actions (source->priv->action_group,
-						   G_OBJECT (shell),
-						   rb_podcast_source_actions,
-						   G_N_ELEMENTS (rb_podcast_source_actions));
+	builder = rb_builder_load ("podcast-popups.ui", NULL);
+	source->priv->feed_popup = G_MENU_MODEL (gtk_builder_get_object (builder, "podcast-feed-popup"));
+	source->priv->episode_popup = G_MENU_MODEL (gtk_builder_get_object (builder, "podcast-episode-popup"));
+	g_object_unref (builder);
 
-	source->priv->add_action = gtk_action_group_get_action (source->priv->action_group,
-								"MusicNewPodcast");
-	/* Translators: this is the toolbar button label
-	   for New Podcast Feed action. */
-	g_object_set (source->priv->add_action, "short-label", C_("Podcast", "Add"), NULL);
-
-	action = gtk_action_group_get_action (source->priv->action_group,
-					      "PodcastFeedUpdate");
-	/* Translators: this is the toolbar button label
-	   for Update Feed action. */
-	g_object_set (action, "short-label", _("Update"), NULL);
-
-	if (gtk_action_group_get_action (source->priv->action_group,
-					 rb_podcast_source_radio_actions[0].name) == NULL) {
-		gtk_action_group_add_radio_actions (source->priv->action_group,
-						    rb_podcast_source_radio_actions,
-						    G_N_ELEMENTS (rb_podcast_source_radio_actions),
-						    0,
-						    NULL,
-						    NULL);
-		rb_source_search_basic_create_for_actions (source->priv->action_group,
-							   rb_podcast_source_radio_actions,
-							   G_N_ELEMENTS (rb_podcast_source_radio_actions));
-	}
-
-	source->priv->default_search = rb_source_search_basic_new (RHYTHMDB_PROP_SEARCH_MATCH);
+	source->priv->default_search = rb_source_search_basic_new (RHYTHMDB_PROP_SEARCH_MATCH, NULL);
 
 	source->priv->paned = gtk_paned_new (GTK_ORIENTATION_VERTICAL);
 
@@ -1499,22 +1503,12 @@ impl_constructed (GObject *object)
 	rb_property_view_set_selection_mode (RB_PROPERTY_VIEW (source->priv->feeds),
 					     GTK_SELECTION_MULTIPLE);
 
-	query_model = rhythmdb_query_model_new_empty (source->priv->db);
-	source->priv->feed_model = rb_property_view_get_model (RB_PROPERTY_VIEW (source->priv->feeds));
-	g_object_set (source->priv->feed_model, "query-model", query_model, NULL);
-
-	/* maybe do this async? */
-	rhythmdb_do_full_query_parsed (source->priv->db,
-				       RHYTHMDB_QUERY_RESULTS (query_model),
-				       source->priv->base_query);
-	g_object_unref (query_model);
-
-	/* error indicator column */
+	/* status indicator column */
 	column = gtk_tree_view_column_new ();
 	renderer = rb_cell_renderer_pixbuf_new ();
 	gtk_tree_view_column_pack_start (column, renderer, TRUE);
 	gtk_tree_view_column_set_cell_data_func (column, renderer,
-						 (GtkTreeCellDataFunc) podcast_feed_error_cell_data_func,
+						 (GtkTreeCellDataFunc) podcast_feed_pixbuf_cell_data_func,
 						 source, NULL);
 	gtk_tree_view_column_set_sizing (column, GTK_TREE_VIEW_COLUMN_FIXED);
 	gtk_tree_view_column_set_fixed_width (column, gdk_pixbuf_get_width (source->priv->error_pixbuf) + 5);
@@ -1523,10 +1517,10 @@ impl_constructed (GObject *object)
 	rb_property_view_append_column_custom (source->priv->feeds, column);
 	g_signal_connect_object (renderer,
 				 "pixbuf-clicked",
-				 G_CALLBACK (podcast_error_pixbuf_clicked_cb),
+				 G_CALLBACK (podcast_status_pixbuf_clicked_cb),
 				 source, 0);
 
-	/* redraw error indicator when errors are set or cleared */
+	/* redraw status when errors are set or cleared */
 	g_signal_connect_object (source->priv->db,
 				 "entry_changed",
 				 G_CALLBACK (podcast_entry_changed_cb),
@@ -1584,8 +1578,23 @@ impl_constructed (GObject *object)
 			   GDK_ACTION_COPY | GDK_ACTION_MOVE);
 
 	/* set up toolbar */
-	source->priv->toolbar = rb_source_toolbar_new (RB_DISPLAY_PAGE (source), ui_manager);
-	rb_source_toolbar_add_search_entry (source->priv->toolbar, "/PodcastSourceSearchMenu", NULL);
+	source->priv->toolbar = rb_source_toolbar_new (RB_DISPLAY_PAGE (source), accel_group);
+
+	source->priv->search_action = rb_source_create_search_action (RB_SOURCE (source));
+	g_action_map_add_action (G_ACTION_MAP (g_application_get_default ()), source->priv->search_action);
+
+	rb_source_search_basic_register (RHYTHMDB_PROP_SEARCH_MATCH, "search-match", _("Search all fields"));
+	rb_source_search_basic_register (RHYTHMDB_PROP_ALBUM_FOLDED, "podcast-feed", _("Search podcast feeds"));
+	rb_source_search_basic_register (RHYTHMDB_PROP_TITLE_FOLDED, "podcast-episode", _("Search podcast episodes"));
+
+	section = g_menu_new ();
+	rb_source_search_add_to_menu (section, "app", source->priv->search_action, "search-match");
+	rb_source_search_add_to_menu (section, "app", source->priv->search_action, "podcast-feed");
+	rb_source_search_add_to_menu (section, "app", source->priv->search_action, "podcast-episode");
+	source->priv->search_popup = G_MENU_MODEL (g_menu_new ());
+	g_menu_append_section (G_MENU (source->priv->search_popup), NULL, G_MENU_MODEL (section));
+
+	rb_source_toolbar_add_search_entry_menu (source->priv->toolbar, source->priv->search_popup, source->priv->search_action);
 
 	/* pack the feed and post views into the source */
 	gtk_paned_pack1 (GTK_PANED (source->priv->paned),
@@ -1626,10 +1635,10 @@ impl_constructed (GObject *object)
 				 GTK_WIDGET (source->priv->feeds));
 
 	g_object_unref (settings);
-	g_object_unref (ui_manager);
+	g_object_unref (accel_group);
 	g_object_unref (shell);
 
-	rb_podcast_source_do_query (source);
+	rb_podcast_source_do_query (source, TRUE);
 }
 
 static void
@@ -1639,30 +1648,13 @@ impl_dispose (GObject *object)
 
 	source = RB_PODCAST_SOURCE (object);
 
-	if (source->priv->db != NULL) {
-		g_object_unref (source->priv->db);
-		source->priv->db = NULL;
-	}
-
-	if (source->priv->search_query != NULL) {
-		rhythmdb_query_free (source->priv->search_query);
-		source->priv->search_query = NULL;
-	}
-
-	if (source->priv->action_group != NULL) {
-		g_object_unref (source->priv->action_group);
-		source->priv->action_group = NULL;
-	}
-
-	if (source->priv->podcast_mgr != NULL) {
-		g_object_unref (source->priv->podcast_mgr);
-		source->priv->podcast_mgr = NULL;
-	}
-
-	if (source->priv->error_pixbuf != NULL) {
-		g_object_unref (source->priv->error_pixbuf);
-		source->priv->error_pixbuf = NULL;
-	}
+	g_clear_object (&source->priv->db);
+	g_clear_object (&source->priv->search_query);
+	g_clear_object (&source->priv->podcast_mgr);
+	g_clear_object (&source->priv->error_pixbuf);
+	g_clear_object (&source->priv->refresh_pixbuf);
+	g_clear_object (&source->priv->search_action);
+	g_clear_object (&source->priv->search_popup);
 
 	G_OBJECT_CLASS (rb_podcast_source_parent_class)->dispose (object);
 }
@@ -1699,10 +1691,15 @@ rb_podcast_source_init (RBPodcastSource *source)
 
 	icon_theme = gtk_icon_theme_get_default ();
 	source->priv->error_pixbuf = gtk_icon_theme_load_icon (icon_theme,
-							       "dialog-error",
+							       "dialog-error-symbolic",
 							       16,
 							       0,
 							       NULL);
+	source->priv->refresh_pixbuf = gtk_icon_theme_load_icon (icon_theme,
+								 "view-refresh-symbolic",
+								 16,
+								 0,
+								 NULL);
 }
 
 static void
@@ -1720,8 +1717,8 @@ rb_podcast_source_class_init (RBPodcastSourceClass *klass)
 
 	page_class->get_status = impl_get_status;
 	page_class->receive_drag = impl_receive_drag;
-	page_class->show_popup = impl_show_popup;
 
+	source_class->reset_filters = impl_reset_filters;
 	source_class->impl_add_to_queue = impl_add_to_queue;
 	source_class->impl_can_add_to_queue = impl_can_add_to_queue;
 	source_class->impl_can_copy = (RBSourceFeatureFunc) rb_false_function;
@@ -1732,8 +1729,7 @@ rb_podcast_source_class_init (RBPodcastSourceClass *klass)
 	source_class->impl_handle_eos = impl_handle_eos;
 	source_class->impl_search = impl_search;
 	source_class->impl_song_properties = impl_song_properties;
-	source_class->impl_get_delete_action = impl_get_delete_action;
-	source_class->impl_reset_filters = impl_reset_filters;
+	source_class->impl_get_delete_label = impl_get_delete_label;
 
 	g_object_class_install_property (object_class,
 					 PROP_PODCAST_MANAGER,
@@ -1748,6 +1744,13 @@ rb_podcast_source_class_init (RBPodcastSourceClass *klass)
 							       "Base query",
 							       "Base query for the source",
 							       G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
+	g_object_class_install_property (object_class,
+					 PROP_SHOW_ALL_FEEDS,
+					 g_param_spec_boolean ("show-all-feeds",
+							       "show-all-feeds",
+							       "show all feeds",
+							       FALSE,
+							       G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
 
 	g_object_class_override_property (object_class, PROP_SHOW_BROWSER, "show-browser");
 
