@@ -56,6 +56,11 @@ static gboolean stop = FALSE;
 
 static gboolean enqueue = FALSE;
 
+static gboolean repeat = FALSE;
+static gboolean no_repeat = FALSE;
+static gboolean shuffle = FALSE;
+static gboolean no_shuffle = FALSE;
+
 static gboolean clear_queue = FALSE;
 
 static gchar *select_source = NULL;
@@ -103,6 +108,11 @@ static GOptionEntry args[] = {
 	{ "activate-source", 0, 0, G_OPTION_ARG_STRING, &activate_source, N_("Activate the source matching the specified URI"), N_("Source to activate")},
 	{ "play-source", 0, 0, G_OPTION_ARG_STRING, &play_source, N_("Play from the source matching the specified URI"), N_("Source to play from")},
 
+	{ "repeat", 0, 0, G_OPTION_ARG_NONE, &repeat, N_("Enable repeat playback order"), NULL },
+	{ "no-repeat", 0, 0, G_OPTION_ARG_NONE, &no_repeat, N_("Disable repeat playback order"), NULL },
+	{ "shuffle", 0, 0, G_OPTION_ARG_NONE, &shuffle, N_("Enable shuffle playback order"), NULL },
+	{ "no-shuffle", 0, 0, G_OPTION_ARG_NONE, &no_shuffle, N_("Disable shuffle playback order"), NULL },
+
 	{ "set-volume", 0, 0, G_OPTION_ARG_DOUBLE, &set_volume, N_("Set the playback volume"), NULL },
 	{ "volume-up", 0, 0, G_OPTION_ARG_NONE, &volume_up, N_("Increase the playback volume"), NULL },
 	{ "volume-down", 0, 0, G_OPTION_ARG_NONE, &volume_down, N_("Decrease the playback volume"), NULL },
@@ -541,7 +551,7 @@ rate_song (GDBusProxy *mpris, gdouble song_rating)
 static void
 state_changed_cb (GActionGroup *action, const char *action_name, GVariant *state, GMainLoop *loop)
 {
-	if (g_strcmp0 (action_name, "LoadURI") == 0) {
+	if (g_strcmp0 (action_name, "load-uri") == 0) {
 		gboolean loaded, scanned;
 
 		g_variant_get (state, "(bb)", &loaded, &scanned);
@@ -562,7 +572,7 @@ state_changed_signal_cb (GDBusProxy *proxy, const char *sender_name, const char
 	}
 
 	g_variant_get (parameters, "(sv)", &action, &state);
-	if (g_strcmp0 (action, "LoadURI") == 0) {
+	if (g_strcmp0 (action, "load-uri") == 0) {
 		GApplication *app;
 		app = g_object_get_data (G_OBJECT (proxy), "actual-app");
 		state_changed_cb (G_ACTION_GROUP (app), action, state, loop);
@@ -653,7 +663,7 @@ main (int argc, char **argv)
 	}
 
 	/* wait until it's ready to accept control */
-	state = g_action_group_get_action_state (G_ACTION_GROUP (app), "LoadURI");
+	state = g_action_group_get_action_state (G_ACTION_GROUP (app), "load-uri");
 	if (state == NULL) {
 		rb_debug ("couldn't get app startup state");
 		exit (0);
@@ -697,7 +707,11 @@ main (int argc, char **argv)
 				       &error);
 	if (mpris == NULL || proxy_has_name_owner (mpris) == FALSE) {
 		g_warning ("MPRIS D-Bus interface not available, some things won't work");
-		if (next || previous || (seek != 0) || play || do_pause || play_pause || stop || volume_up || volume_down || (set_volume > -0.01)) {
+		if (next   || previous  || (seek != 0) ||
+		    play   || do_pause  || play_pause  ||
+		    stop   || volume_up || volume_down ||
+		    repeat || no_repeat || shuffle     ||
+		    no_shuffle || (set_volume > -0.01)) {
 			exit (1);
 		}
 	}
@@ -730,6 +744,7 @@ main (int argc, char **argv)
 	    play_uri || other_stuff ||
 	    play || do_pause || play_pause || stop ||
 	    print_playing || print_playing_format ||
+	    repeat || no_repeat || shuffle || no_shuffle ||
 	    (set_volume > -0.01) || volume_up || volume_down || print_volume /*|| mute || unmute*/ || (set_rating > -0.01))
 		no_present = TRUE;
 
@@ -745,6 +760,48 @@ main (int argc, char **argv)
 		rate_song (mpris, set_rating);
 	}
 
+	if (repeat) {
+		g_dbus_proxy_call_sync (mpris,
+					"org.freedesktop.DBus.Properties.Set",
+					g_variant_new ("(ssv)", "org.mpris.MediaPlayer2.Player", "LoopStatus", g_variant_new_string ("Playlist")),
+					G_DBUS_CALL_FLAGS_NONE,
+					-1,
+					NULL,
+					&error);
+		annoy (&error);
+
+	} else if (no_repeat) {
+		g_dbus_proxy_call_sync (mpris,
+					"org.freedesktop.DBus.Properties.Set",
+					g_variant_new ("(ssv)", "org.mpris.MediaPlayer2.Player", "LoopStatus", g_variant_new_string ("None")),
+					G_DBUS_CALL_FLAGS_NONE,
+					-1,
+					NULL,
+					&error);
+		annoy (&error);
+	}
+
+	if (shuffle) {
+		g_dbus_proxy_call_sync (mpris,
+					"org.freedesktop.DBus.Properties.Set",
+					g_variant_new ("(ssv)", "org.mpris.MediaPlayer2.Player", "Shuffle", g_variant_new_boolean (TRUE)),
+					G_DBUS_CALL_FLAGS_NONE,
+					-1,
+					NULL,
+					&error);
+		annoy (&error);
+
+	} else if (no_shuffle) {
+		g_dbus_proxy_call_sync (mpris,
+					"org.freedesktop.DBus.Properties.Set",
+					g_variant_new ("(ssv)", "org.mpris.MediaPlayer2.Player", "Shuffle", g_variant_new_boolean (FALSE)),
+					G_DBUS_CALL_FLAGS_NONE,
+					-1,
+					NULL,
+					&error);
+		annoy (&error);
+	}
+
 	/* skip to next or previous track */
 	if (next) {
 		rb_debug ("next track");
@@ -801,7 +858,7 @@ main (int argc, char **argv)
 				annoy (&error);
 			} else {
 				rb_debug ("importing %s", fileuri);
-				g_action_group_activate_action (G_ACTION_GROUP (app), "LoadURI", g_variant_new ("(sb)", fileuri, FALSE));
+				g_action_group_activate_action (G_ACTION_GROUP (app), "load-uri", g_variant_new ("(sb)", fileuri, FALSE));
 			}
 			g_free (fileuri);
 			g_object_unref (file);
@@ -811,13 +868,13 @@ main (int argc, char **argv)
 	/* select/activate/play source */
 	if (select_source) {
 		rb_debug ("selecting source %s", select_source);
-		g_action_group_activate_action (G_ACTION_GROUP (app), "ActivateSource", g_variant_new ("(su)", select_source, 0));
+		g_action_group_activate_action (G_ACTION_GROUP (app), "activate-source", g_variant_new ("(su)", select_source, 0));
 	} else if (activate_source) {
 		rb_debug ("activating source %s", activate_source);
-		g_action_group_activate_action (G_ACTION_GROUP (app), "ActivateSource", g_variant_new ("(su)", activate_source, 1));
+		g_action_group_activate_action (G_ACTION_GROUP (app), "activate-source", g_variant_new ("(su)", activate_source, 1));
 	} else if (play_source) {
 		rb_debug ("playing source %s", play_source);
-		g_action_group_activate_action (G_ACTION_GROUP (app), "ActivateSource", g_variant_new ("(su)", play_source, 2));
+		g_action_group_activate_action (G_ACTION_GROUP (app), "activate-source", g_variant_new ("(su)", play_source, 2));
 	}
 
 	/* play uri */
@@ -831,7 +888,7 @@ main (int argc, char **argv)
 			g_warning ("couldn't convert \"%s\" to a URI", play_uri);
 		} else {
 			rb_debug ("loading and playing %s", fileuri);
-			g_action_group_activate_action (G_ACTION_GROUP (app), "LoadURI", g_variant_new ("(sb)", fileuri, TRUE));
+			g_action_group_activate_action (G_ACTION_GROUP (app), "load-uri", g_variant_new ("(sb)", fileuri, TRUE));
 			annoy (&error);
 		}
 		g_free (fileuri);
