@@ -171,6 +171,10 @@ typedef enum
 } RhythmDBPropType;
 
 enum {
+	RHYTHMDB_PODCAST_FEED_STATUS_HIDDEN = 0,
+	RHYTHMDB_PODCAST_FEED_STATUS_NORMAL = 1,
+	RHYTHMDB_PODCAST_FEED_STATUS_UPDATING = 2,
+
 	RHYTHMDB_PODCAST_STATUS_COMPLETE = 100,
 	RHYTHMDB_PODCAST_STATUS_ERROR = 101,
 	RHYTHMDB_PODCAST_STATUS_WAITING = 102,
