@@ -63,6 +63,7 @@
 
 static void rb_header_class_init (RBHeaderClass *klass);
 static void rb_header_init (RBHeader *header);
+static void rb_header_constructed (GObject *object);
 static void rb_header_dispose (GObject *object);
 static void rb_header_finalize (GObject *object);
 static void rb_header_set_property (GObject *object,
@@ -97,6 +98,8 @@ static void uri_dropped_cb (RBFadingImage *image, const char *uri, RBHeader *hea
 static void pixbuf_dropped_cb (RBFadingImage *image, GdkPixbuf *pixbuf, RBHeader *header);
 static void image_button_press_cb (GtkWidget *widget, GdkEvent *event, RBHeader *header);
 static void art_added_cb (RBExtDB *db, RBExtDBKey *key, const char *filename, GValue *data, RBHeader *header);
+static void volume_widget_changed_cb (GtkScaleButton *widget, gdouble volume, RBHeader *header);
+static void player_volume_changed_cb (RBShellPlayer *player, GParamSpec *pspec, RBHeader *header);
 
 struct RBHeaderPrivate
 {
@@ -105,11 +108,15 @@ struct RBHeaderPrivate
 	RBExtDB *art_store;
 
 	RBShellPlayer *shell_player;
+	RBSource *playing_source;
+	gulong status_changed_id;
+	gboolean showing_playback_status;
 
 	GtkWidget *songbox;
 	GtkWidget *song;
 	GtkWidget *details;
 	GtkWidget *image;
+	GtkWidget *volume_button;
 
 	GtkWidget *scale;
 	GtkAdjustment *adjustment;
@@ -129,6 +136,8 @@ struct RBHeaderPrivate
 	char *image_path;
 	gboolean show_album_art;
 	gboolean show_slider;
+	
+	gboolean syncing_volume;
 };
 
 enum
@@ -162,6 +171,7 @@ rb_header_class_init (RBHeaderClass *klass)
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
 	GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (klass);
 
+	object_class->constructed = rb_header_constructed;
 	object_class->dispose = rb_header_dispose;
 	object_class->finalize = rb_header_finalize;
 
@@ -269,6 +279,14 @@ static void
 rb_header_init (RBHeader *header)
 {
 	header->priv = G_TYPE_INSTANCE_GET_PRIVATE (header, RB_TYPE_HEADER, RBHeaderPrivate);
+}
+
+static void
+rb_header_constructed (GObject *object)
+{
+	RBHeader *header = RB_HEADER (object);
+
+	RB_CHAIN_GOBJECT_METHOD (rb_header_parent_class, constructed, object);
 
 	gtk_grid_set_column_spacing (GTK_GRID (header), 6);
 	gtk_grid_set_column_homogeneous (GTK_GRID (header), TRUE);
@@ -277,6 +295,9 @@ rb_header_init (RBHeader *header)
 	/* set up position slider */
 	header->priv->adjustment = GTK_ADJUSTMENT (gtk_adjustment_new (0.0, 0.0, 10.0, 1.0, 10.0, 0.0));
 	header->priv->scale = gtk_scale_new (GTK_ORIENTATION_HORIZONTAL, header->priv->adjustment);
+	gtk_range_set_fill_level (GTK_RANGE (header->priv->scale), 0.0);
+	gtk_range_set_show_fill_level (GTK_RANGE (header->priv->scale), FALSE);
+	gtk_range_set_restrict_to_fill_level (GTK_RANGE (header->priv->scale), FALSE);
 	gtk_widget_set_hexpand (header->priv->scale, TRUE);
 	g_signal_connect_object (G_OBJECT (header->priv->scale),
 				 "button_press_event",
@@ -360,10 +381,20 @@ rb_header_init (RBHeader *header)
 			  G_CALLBACK (image_button_press_cb),
 			  header);
 
+	/* volume button */
+	header->priv->volume_button = gtk_volume_button_new ();
+	g_signal_connect (header->priv->volume_button, "value-changed",
+			  G_CALLBACK (volume_widget_changed_cb),
+			  header);
+	g_signal_connect (header->priv->shell_player, "notify::volume",
+			  G_CALLBACK (player_volume_changed_cb),
+			  header);
+
 	gtk_grid_attach (GTK_GRID (header), header->priv->image, 0, 0, 1, 1);
 	gtk_grid_attach (GTK_GRID (header), header->priv->songbox, 2, 0, 1, 1);
 	gtk_grid_attach (GTK_GRID (header), header->priv->timebutton, 3, 0, 1, 1);
 	gtk_grid_attach (GTK_GRID (header), header->priv->scale, 4, 0, 1, 1);
+	gtk_grid_attach (GTK_GRID (header), header->priv->volume_button, 5, 0, 1, 1);
 
 	/* currently, nothing sets this.  it should be set on track changes. */
 	header->priv->seekable = TRUE;
@@ -442,6 +473,11 @@ art_added_cb (RBExtDB *db, RBExtDBKey *key, const char *filename, GValue *data,
 	art_cb (key, filename, data, header);
 }
 
+static void
+playback_status_changed_cb (RBSource *source, RBHeader *header)
+{
+	rb_header_sync (header);
+}
 
 static void
 rb_header_playing_song_changed_cb (RBShellPlayer *player, RhythmDBEntry *entry, RBHeader *header)
@@ -449,6 +485,10 @@ rb_header_playing_song_changed_cb (RBShellPlayer *player, RhythmDBEntry *entry,
 	if (header->priv->entry == entry)
 		return;
 
+	if (header->priv->entry != NULL) {
+		g_signal_handler_disconnect (header->priv->playing_source, header->priv->status_changed_id);
+	}
+
 	rb_fading_image_start (RB_FADING_IMAGE (header->priv->image), 2000);
 
 	header->priv->entry = entry;
@@ -465,6 +505,13 @@ rb_header_playing_song_changed_cb (RBShellPlayer *player, RhythmDBEntry *entry,
 				   g_object_ref (header),
 				   g_object_unref);
 		rb_ext_db_key_free (key);
+
+		header->priv->playing_source = rb_shell_player_get_playing_source (player);
+		header->priv->status_changed_id =
+			g_signal_connect (header->priv->playing_source,
+					  "playback-status-changed",
+					  G_CALLBACK (playback_status_changed_cb),
+					  header);
 	} else {
 		header->priv->duration = 0;
 	}
@@ -498,6 +545,7 @@ rb_header_size_allocate (GtkWidget *widget, GtkAllocation *allocation)
 	int info_width;
 	int time_width;
 	int image_width;
+	int volume_width;
 	GtkAllocation child_alloc;
 	gboolean rtl;
 
@@ -524,6 +572,15 @@ rb_header_size_allocate (GtkWidget *widget, GtkAllocation *allocation)
 		image_width = 0;
 	}
 
+	/* allocate space for the volume button at the end */
+	gtk_widget_get_preferred_width (RB_HEADER (widget)->priv->volume_button, &volume_width, NULL);
+	child_alloc.x = (allocation->x + allocation->width) - volume_width;
+	child_alloc.y = allocation->y;
+	child_alloc.width = volume_width;
+	child_alloc.height = allocation->height;
+	allocation->width -= volume_width + spacing;
+	gtk_widget_size_allocate (RB_HEADER (widget)->priv->volume_button, &child_alloc);
+
 	/* figure out how much space to allocate to the scale.
 	 * it gets at least its minimum size, at most 1/3 of the
 	 * space we have.
@@ -726,13 +783,6 @@ static void
 rb_header_sync (RBHeader *header)
 {
 	char *label_text;
-	const char *location = "<null>";
-
-	if (header->priv->entry != NULL) {
-		location = rhythmdb_entry_get_string (header->priv->entry, RHYTHMDB_PROP_LOCATION);
-	}
-	rb_debug ("syncing with entry = %s", location);
-
 	if (header->priv->entry != NULL) {
 		const char *title;
 		const char *album;
@@ -743,6 +793,8 @@ rb_header_sync (RBHeader *header)
 		char *streaming_album;
 		PangoDirection widget_dir;
 
+		rb_debug ("syncing with %s",
+			  rhythmdb_entry_get_string (header->priv->entry, RHYTHMDB_PROP_LOCATION));
 		gboolean have_duration = (header->priv->duration > 0);
 
 		title = rhythmdb_entry_get_string (header->priv->entry, RHYTHMDB_PROP_TITLE);
@@ -824,6 +876,27 @@ rb_header_sync (RBHeader *header)
 			g_free (t);
 		}
 
+		if (header->priv->playing_source) {
+			char *text = NULL;
+			float progress = 0.0;
+
+			rb_source_get_playback_status (header->priv->playing_source, &text, &progress);
+			if (text) {
+				header->priv->showing_playback_status = TRUE;
+				gtk_widget_show (header->priv->timelabel);
+				gtk_widget_show (header->priv->timebutton);
+				gtk_label_set_text (GTK_LABEL (header->priv->timelabel), text);
+				g_free (text);
+			} else {
+				header->priv->showing_playback_status = FALSE;
+			}
+			gtk_range_set_show_fill_level (GTK_RANGE (header->priv->scale), (text != NULL));
+
+			progress = progress * gtk_adjustment_get_upper (header->priv->adjustment);
+			gtk_range_set_fill_level (GTK_RANGE (header->priv->scale), progress);
+		}
+
+
 		gtk_widget_set_sensitive (header->priv->scale, have_duration && header->priv->seekable);
 		rb_header_sync_time (header);
 
@@ -839,6 +912,7 @@ rb_header_sync (RBHeader *header)
 		gtk_label_set_text (GTK_LABEL (header->priv->details), "");
 
 		rb_header_sync_time (header);
+		gtk_widget_set_sensitive (header->priv->scale, FALSE);
 	}
 }
 
@@ -870,7 +944,7 @@ rb_header_sync_time (RBHeader *header)
 
 		g_object_freeze_notify (G_OBJECT (header->priv->adjustment));
 		gtk_adjustment_set_value (header->priv->adjustment, 0.0);
-		gtk_adjustment_set_upper (header->priv->adjustment, 0.0);
+		gtk_adjustment_set_upper (header->priv->adjustment, 1.0);
 		g_object_thaw_notify (G_OBJECT (header->priv->adjustment));
 
 		header->priv->slider_locked = FALSE;
@@ -885,6 +959,8 @@ slider_press_callback (GtkWidget *widget,
 		       GdkEventButton *event,
 		       RBHeader *header)
 {
+	int height;
+
 	header->priv->slider_dragging = TRUE;
 	header->priv->slider_drag_moved = FALSE;
 	header->priv->latest_set_time = -1;
@@ -899,6 +975,10 @@ slider_press_callback (GtkWidget *widget,
 	event->button = 2;
 #endif
 
+	/* more hack: pretend the trough is at least 20 pixels high */
+	height = gtk_widget_get_allocated_height (widget);
+	if (abs (event->y - (height / 2)) < 10)
+		event->y = height / 2;
 
 	return FALSE;
 }
@@ -1054,12 +1134,16 @@ rb_header_update_elapsed (RBHeader *header)
 	char *duration;
 	char *label;
 
+	if (header->priv->showing_playback_status)
+		return;
+
 	if (header->priv->entry == NULL) {
 		gtk_label_set_text (GTK_LABEL (header->priv->timelabel), "");
 		gtk_widget_hide (header->priv->timelabel);
 		return;
 	}
 	gtk_widget_show (header->priv->timelabel);
+	gtk_widget_show (header->priv->timebutton);
 
 	seconds = header->priv->elapsed_time / RB_PLAYER_SECOND;
 	if (header->priv->duration == 0) {
@@ -1078,7 +1162,6 @@ rb_header_update_elapsed (RBHeader *header)
 
 		/* Translators: remaining time / total time */
 		label = g_strdup_printf (_("-%s / %s"), elapsed, duration);
-		gtk_widget_show (header->priv->timebutton);
 		gtk_label_set_text (GTK_LABEL (header->priv->timelabel), label);
 
 		g_free (elapsed);
@@ -1144,7 +1227,7 @@ pixbuf_dropped_cb (RBFadingImage *image, GdkPixbuf *pixbuf, RBHeader *header)
 	rb_ext_db_key_add_field (key, "artist", artist);
 
 	g_value_init (&v, GDK_TYPE_PIXBUF);
-	g_value_set_object (&v, image);
+	g_value_set_object (&v, pixbuf);
 	rb_ext_db_store (header->priv->art_store, key, RB_EXT_DB_SOURCE_USER_EXPLICIT, &v);
 	g_value_unset (&v);
 
@@ -1206,3 +1289,22 @@ time_button_clicked_cb (GtkWidget *widget, RBHeader *header)
 {
 	g_object_set (header, "show-remaining", !header->priv->show_remaining, NULL);
 }
+
+static void
+volume_widget_changed_cb (GtkScaleButton *vol, gdouble value, RBHeader *header)
+{
+	if (!header->priv->syncing_volume) {
+		g_object_set (header->priv->shell_player, "volume", value, NULL);
+	}
+}
+
+static void
+player_volume_changed_cb (RBShellPlayer *player, GParamSpec *pspec, RBHeader *header)
+{
+	float volume;
+
+	g_object_get (player, "volume", &volume, NULL);
+	header->priv->syncing_volume = TRUE;
+	gtk_scale_button_set_value (GTK_SCALE_BUTTON (header->priv->volume_button), volume);
+	header->priv->syncing_volume = FALSE;
+}
